Package: ProcessCapabilityR
Type: Package
Title: Classical and Generalized Process Capability Indices
Version: 0.1.0
Authors@R: c(
    person("Shikhar", "Tyagi",
           email = "shikhar1093tyagi@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-1606-0844")),
    person("Sumit", "Kumar",
           email = "stats.sumitbhal@gmail.com",
           role = "aut"),
    person("Vrijesh", "Tripathi",
           email = "vrijesh.tripathi@uwi.edu",
           role = "aut")
    )
Maintainer: Shikhar Tyagi <shikhar1093tyagi@gmail.com>
Description: Computes classical process capability indices (Cp, Cpk, Cpu, Cpl,
    Cpm, Cpmk, Pp, Ppk, Ppu, Ppl, Z) and the generalized process capability
    index Cpy (Maiti, Saha & Nanda, 2010) <doi:10.1080/16843703.2010.11673233>
    for any continuous or discrete quality characteristic. Users supply the
    probability density function (PDF) and cumulative distribution function (CDF)
    of the characteristic, and the package returns point estimates, bootstrap
    confidence intervals (percentile and BCa), and sensitivity tables/plots
    across ranges of short-term standard deviation (sigma), long-term standard
    deviation (s), desired yield (p0), and significance levels. Classical indices
    are recoverable as special cases under the normal distribution. The package
    follows the theory and notation of Kane (1986) <doi:10.1080/00224065.1986.11978984>,
    Chan, Cheng & Spiring (1988) <doi:10.1080/00224065.1988.11979102>,
    Pearn, Kotz & Johnson (1992) <doi:10.1080/00224065.1992.11979403>,
    Kotz & Johnson (2002) <doi:10.1080/00224065.2002.11980119>,
    Montgomery (2020, ISBN:978-1-119-39930-8), Juran (1974, ISBN:978-0-07-033176-1),
    Harry & Schroeder (2000, ISBN:978-0-385-49437-2), and the AIAG SPC Reference
    Manual (2005, ISBN:978-1-60534-026-3).
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5.0)
Imports: stats
Suggests: ggplot2 (>= 3.0.0), testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-23 17:54:01 UTC; 30017827
Author: Shikhar Tyagi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1606-0844>),
  Sumit Kumar [aut],
  Vrijesh Tripathi [aut]
Repository: CRAN
Date/Publication: 2026-08-03 18:00:24 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,pci_grid)
S3method(print,pci_ci)
S3method(print,pci_dist)
export(cp)
export(cpk)
export(cpl)
export(cpm)
export(cpmk)
export(cpu)
export(cpy)
export(pci)
export(pci_ci)
export(pci_dist)
export(pci_dist_normal)
export(pci_grid)
export(pp)
export(ppk)
export(ppl)
export(ppu)
export(z_level)

# ProcessCapabilityR

<!-- badges: start -->
<!-- badges: end -->

**Classical and Generalized Process Capability Indices for Any Distribution**

## Overview

**ProcessCapabilityR** computes Process Capability Indices (PCIs) for any
quality characteristic — not just the normal distribution — by letting users
supply the characteristic's PDF and CDF directly.

### Features

- **11 classical indices**: Cp, Cpk, Cpu, Cpl, Cpm, Cpmk, Pp, Ppk, Ppu,
  Ppl, Z (sigma level)
- **Generalized index Cpy** (Maiti, Saha & Nanda, 2010) for *any*
  continuous or discrete distribution
- **Bootstrap confidence intervals** — percentile and BCa methods,
  parametric or nonparametric
- **Sensitivity grids** across σ, s, p₀ values at multiple significance
  levels
- **ggplot2 visualisation** of sensitivity sweeps with confidence bands

## Installation

```r
# Install from GitHub
# devtools::install_github("shikhartyagi/ProcessCapabilityR")

# Or install locally from source
devtools::install("path/to/ProcessCapabilityR")

# Load the package
library(ProcessCapabilityR)
```

## Quick Start

### Classical Indices (Normal Process)

```r
library(ProcessCapabilityR)

# Process: USL = 63, LSL = 57, μ = 60, σ = 1 (centered)
cp(LSL = 57, USL = 63, sigma = 1)                        # 1.0
cpk(LSL = 57, USL = 63, mu = 60, sigma = 1)              # 1.0
cpm(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60) # 1.0
z_level(LSL = 57, USL = 63, mu = 60, sigma = 1)          # 3.0

# Or use the generic interface
pci("Cp",  LSL = 57, USL = 63, sigma = 1)
pci("Cpk", LSL = 57, USL = 63, mu = 60, sigma = 1)

# Performance indices (long-term)
pp(LSL = 57, USL = 63, s = 1.5)                          # 0.667
ppk(LSL = 57, USL = 63, xbar = 60, s = 1.5)              # 0.667
```

### Generalized Cpy (Non-Normal Distribution)

```r
# Define a Weibull distribution
dist_weibull <- pci_dist(
  pdf    = function(x, shape, scale) dweibull(x, shape, scale),
  cdf    = function(x, shape, scale) pweibull(x, shape, scale),
  params = list(shape = 2, scale = 10),
  support = c(0, 50)
)

# Compute Cpy with desired yield p₀ = 0.95
pci("Cpy", dist = dist_weibull, LSL = 2, USL = 20, p0 = 0.95)
```

### Bootstrap Confidence Intervals

```r
dist_norm <- pci_dist_normal(mean = 60, sd = 1)
ci <- pci_ci("Cp", dist = dist_norm, n = 30,
             LSL = 57, USL = 63, alpha = 0.05, B = 2000)
print(ci)
```

### Sensitivity Grid & Plot

```r
grid <- pci_grid("Cp",
                 dist = pci_dist_normal(60, 1),
                 LSL = 57, USL = 63,
                 sigma_vals = seq(0.5, 2.0, by = 0.1),
                 mu = 60,
                 alpha_vals = c(0.10, 0.05, 0.01),
                 n = 30, B = 500)
plot(grid, x_axis = "sigma")
```

### Cpy Sensitivity Grid

```r
grid_cpy <- pci_grid("Cpy",
                     dist = pci_dist_normal(60, 1),
                     LSL = 57, USL = 63,
                     p0_vals = c(0.90, 0.95, 0.99),
                     alpha_vals = c(0.10, 0.05, 0.01),
                     n = 30, B = 500)
plot(grid_cpy, x_axis = "p0")
```

## Available Indices

| Index | Formula | Description |
|-------|---------|-------------|
| Cp    | (USL − LSL) / (6σ) | Process potential |
| Cpk   | min[(USL − μ)/(3σ), (μ − LSL)/(3σ)] | Capability with centering |
| Cpu   | (USL − μ) / (3σ) | Upper capability |
| Cpl   | (μ − LSL) / (3σ) | Lower capability |
| Cpm   | (USL − LSL) / (6√(σ² + (μ−T)²)) | Taguchi (target-sensitive) |
| Cpmk  | min[(USL−μ), (μ−LSL)] / (3√(σ²+(μ−T)²)) | Modified Taguchi |
| Pp    | (USL − LSL) / (6s) | Long-term performance |
| Ppk   | min[(USL − x̄)/(3s), (x̄ − LSL)/(3s)] | Performance with centering |
| Ppu   | (USL − x̄) / (3s) | Upper performance |
| Ppl   | (x̄ − LSL) / (3s) | Lower performance |
| Z     | min[(USL − μ)/σ, (μ − LSL)/σ] | Sigma level |
| Cpy   | [F(USL)−F(LSL)] / [F(UDL)−F(LDL)] | Generalized (any distribution) |

## References

- Juran, J.M. (1974). *Quality Control Handbook* (3rd ed.). McGraw-Hill.
- Kane, V.E. (1986). Process capability indices. *Journal of Quality
  Technology*, 18(1), 41–52.
- Chan, L.K., Cheng, S.W., & Spiring, F.A. (1988). A new measure of
  process capability: Cpm. *Journal of Quality Technology*, 20(3), 162–175.
- Pearn, W.L., Kotz, S., & Johnson, N.L. (1992). Distributional and
  inferential properties of process capability indices. *Journal of Quality
  Technology*, 24(4), 216–231.
- Harry, M., & Schroeder, R. (2000). *Six Sigma: The Breakthrough
  Management Strategy*. Doubleday.
- Kotz, S., & Johnson, N.L. (2002). Process capability indices — a review,
  1992–2000. *Journal of Quality Technology*, 34(1), 2–19.
- AIAG (2005). *Statistical Process Control (SPC) Reference Manual*
  (2nd ed.).
- Maiti, S.S., Saha, M., & Nanda, A.K. (2010). On generalizing process
  capability indices. *Quality Technology & Quantitative Management*,
  7(3), 279–300.
- Montgomery, D.C. (2020). *Introduction to Statistical Quality Control*
  (8th ed.). Wiley.

## Authors

- **Shikhar Tyagi** — maintainer
  ([ORCID](https://orcid.org/0000-0003-1606-0844))
  \<shikhar1093tyagi@gmail.com\>
- **Sumit Kumar** \<stats.sumitbhal@gmail.com\>
- **Vrijesh Tripathi** \<vrijesh.tripathi@uwi.edu\>

## License

MIT © 2025 Shikhar Tyagi, Sumit Kumar, Vrijesh Tripathi

#' Sensitivity Grid of Process Capability Indices
#'
#' Computes a PCI across a grid of parameter values (short-term standard
#' deviation \code{sigma}, long-term standard deviation \code{s}, and/or
#' desired yield \code{p0}), optionally with bootstrap confidence intervals at
#' multiple significance levels.
#'
#' @param index Character: one of the 12 index names (see \code{\link{pci}}).
#' @param dist A \code{\link{pci_dist}} object (required for \code{"Cpy"}).
#' @param LSL,USL Specification limits.
#' @param target Target value (for \code{Cpm}/\code{Cpmk}).
#' @param sigma_vals Numeric vector of \eqn{\sigma} values to sweep
#'   (for capability indices).
#' @param s_vals Numeric vector of \eqn{s} values to sweep
#'   (for performance indices).
#' @param p0_vals Numeric vector of \eqn{p_0} values to sweep (for \code{Cpy}).
#' @param mu Short-term process mean.
#' @param xbar Long-term process mean.
#' @param alpha_vals Numeric vector of significance levels.
#'   Default \code{c(0.10, 0.05, 0.01)}.
#' @param n Sample size for bootstrap CIs. If \code{NULL}, CIs are omitted.
#' @param B Bootstrap replications. Default 2000.
#'
#' @return A \code{data.frame} (with additional class \code{"pci_grid"})
#'   in long format with columns:
#'   \code{index}, \code{sigma}, \code{s}, \code{p0}, \code{alpha},
#'   \code{estimate}, \code{lower}, \code{upper}.
#'   Columns that do not apply to the chosen index contain \code{NA}.
#'
#' @examples
#' grid <- pci_grid("Cp", LSL = 57, USL = 63,
#'                  sigma_vals = c(0.5, 1.0, 1.5),
#'                  mu = 60, alpha_vals = c(0.05, 0.01))
#' print(grid)
#'
#' @seealso \code{\link{pci}}, \code{\link{pci_ci}},
#'   \code{\link{plot.pci_grid}}
#' @export
pci_grid <- function(index,
                     dist       = NULL,
                     LSL        = NULL, USL  = NULL, target = NULL,
                     sigma_vals = NULL,
                     s_vals     = NULL,
                     p0_vals    = NULL,
                     mu         = NULL, xbar = NULL,
                     alpha_vals = c(0.10, 0.05, 0.01),
                     n          = NULL,
                     B          = 2000) {

  valid_indices <- c("Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk",
                     "Pp", "Ppk", "Ppu", "Ppl", "Z", "Cpy")
  index <- match.arg(index, valid_indices)

  # Determine which sweep variable applies
  capability_idx  <- c("Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk", "Z")
  performance_idx <- c("Pp", "Ppk", "Ppu", "Ppl")

  # Build the sweep vectors
  if (index %in% capability_idx) {
    sweep_var <- "sigma"
    sweep_vals <- if (!is.null(sigma_vals)) sigma_vals else {
      if (!is.null(s_vals)) s_vals else 1
    }
  } else if (index %in% performance_idx) {
    sweep_var <- "s"
    sweep_vals <- if (!is.null(s_vals)) s_vals else {
      if (!is.null(sigma_vals)) sigma_vals else 1
    }
  } else {
    # Cpy
    sweep_var <- "p0"
    sweep_vals <- if (!is.null(p0_vals)) p0_vals else 0.95
  }

  # Build the result rows
  rows <- list()
  row_idx <- 0L

  for (sv in sweep_vals) {
    # Compute point estimate
    est <- tryCatch({
      switch(sweep_var,
        sigma = pci(index, dist = dist, LSL = LSL, USL = USL, target = target,
                    mu = mu, sigma = sv, xbar = xbar, s = sv),
        s     = pci(index, dist = dist, LSL = LSL, USL = USL, target = target,
                    mu = mu, sigma = sv, xbar = xbar, s = sv),
        p0    = pci(index, dist = dist, LSL = LSL, USL = USL, target = target,
                    mu = mu, sigma = NULL, xbar = xbar, s = NULL,
                    p0 = sv)
      )
    }, error = function(e) NA_real_)

    for (av in alpha_vals) {
      # Compute CI if n is supplied
      ci_lower <- NA_real_
      ci_upper <- NA_real_

      if (!is.null(n) && !is.null(dist)) {
        tryCatch({
          ci_obj <- pci_ci(index = index, dist = dist, n = n,
                           LSL = LSL, USL = USL, target = target,
                           LDL = NULL, UDL = NULL,
                           p0 = if (sweep_var == "p0") sv else NULL,
                           alpha = av, B = B)
          ci_lower <- ci_obj$lower
          ci_upper <- ci_obj$upper
        }, error = function(e) {
          # Leave as NA
        })
      }

      row_idx <- row_idx + 1L
      rows[[row_idx]] <- data.frame(
        index    = index,
        sigma    = if (sweep_var == "sigma") sv else NA_real_,
        s        = if (sweep_var == "s")     sv else NA_real_,
        p0       = if (sweep_var == "p0")    sv else NA_real_,
        alpha    = av,
        estimate = est,
        lower    = ci_lower,
        upper    = ci_upper,
        stringsAsFactors = FALSE
      )
    }
  }

  result <- do.call(rbind, rows)
  rownames(result) <- NULL

  structure(result, class = c("pci_grid", "data.frame"))
}

# tests/testthat/test-pci_grid.R
# Test pci_grid output structure and values.

test_that("pci_grid returns correct structure (no CI)", {
  grid <- pci_grid("Cp",
                   LSL = 57, USL = 63,
                   sigma_vals = c(0.5, 1.0, 1.5),
                   mu = 60,
                   alpha_vals = c(0.05, 0.01))

  expect_s3_class(grid, "pci_grid")
  expect_s3_class(grid, "data.frame")

  # Expected columns
  expected_cols <- c("index", "sigma", "s", "p0", "alpha",
                     "estimate", "lower", "upper")
  expect_equal(names(grid), expected_cols)

  # 3 sigma_vals x 2 alpha_vals = 6 rows
  expect_equal(nrow(grid), 6)

  # All index values should be "Cp"
  expect_true(all(grid$index == "Cp"))

  # s and p0 should be NA
  expect_true(all(is.na(grid$s)))
  expect_true(all(is.na(grid$p0)))

  # lower and upper should be NA (n not supplied)
  expect_true(all(is.na(grid$lower)))
  expect_true(all(is.na(grid$upper)))
})

test_that("pci_grid computes correct estimates for Cp", {
  grid <- pci_grid("Cp",
                   LSL = 57, USL = 63,
                   sigma_vals = c(0.5, 1.0, 1.5),
                   mu = 60,
                   alpha_vals = c(0.05))

  estimates <- unique(grid$estimate)
  expected  <- c(2.0, 1.0, 6 / 9)
  expect_equal(sort(estimates), sort(expected), tolerance = 1e-4)
})

test_that("pci_grid with n produces CI bounds", {
  skip_on_cran()  # bootstrap is slow
  d <- pci_dist_normal(mean = 60, sd = 1)

  grid <- pci_grid("Cp",
                   dist = d,
                   LSL = 57, USL = 63,
                   sigma_vals = c(1.0),
                   mu = 60,
                   alpha_vals = c(0.05),
                   n = 30, B = 100)

  expect_equal(nrow(grid), 1)
  expect_false(is.na(grid$lower))
  expect_false(is.na(grid$upper))
  expect_true(grid$lower <= grid$upper)
})

test_that("pci_grid works for Cpy with p0_vals", {
  d <- pci_dist_normal(mean = 60, sd = 1)

  grid <- pci_grid("Cpy",
                   dist = d,
                   LSL = 57, USL = 63,
                   p0_vals = c(0.90, 0.95, 0.99),
                   alpha_vals = c(0.05))

  expect_equal(nrow(grid), 3)
  expect_true(all(!is.na(grid$estimate)))
  expect_true(all(!is.na(grid$p0)))
  expect_true(all(is.na(grid$sigma)))
  expect_true(all(is.na(grid$s)))
})

# Avoid R CMD check note
if (getRversion() >= "2.15.1") utils::globalVariables(c(".data"))

#' Plot a Process Capability Sensitivity Grid
#'
#' Produces a \pkg{ggplot2} line-and-ribbon plot showing the PCI estimate
#' and confidence bands as a function of the chosen sweep variable, faceted
#' by significance level.
#'
#' @param x A \code{\link{pci_grid}} object.
#' @param x_axis Character: which sweep variable to place on the x-axis.
#'   One of \code{"sigma"}, \code{"s"}, or \code{"p0"}.
#' @param facet_by Character: column name to facet by. Default \code{"alpha"}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns a \code{\link[ggplot2]{ggplot}} object representing
#'   the sensitivity plot. Called primarily for its side effect of rendering a
#'   line-and-ribbon plot to the active graphic device.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   grid <- pci_grid("Cp", LSL = 57, USL = 63,
#'                    sigma_vals = seq(0.5, 2, 0.1), mu = 60,
#'                    alpha_vals = c(0.05, 0.01),
#'                    n = 30, B = 200,
#'                    dist = pci_dist_normal(60, 1))
#'   plot(grid, x_axis = "sigma")
#' }
#' }
#'
#' @seealso \code{\link{pci_grid}}
#' @export
plot.pci_grid <- function(x, x_axis = c("sigma", "s", "p0"),
                          facet_by = "alpha", ...) {

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plotting. ",
         "Install it with install.packages(\"ggplot2\").",
         call. = FALSE)
  }

  x_axis <- match.arg(x_axis)

  # Filter to rows where the x-axis variable is not NA
  plot_data <- x[!is.na(x[[x_axis]]), , drop = FALSE]

  if (nrow(plot_data) == 0) {
    stop("No rows with non-NA values for '", x_axis, "'.")
  }

  # Convert facet variable to factor for nice labelling
  if (facet_by %in% names(plot_data)) {
    plot_data[[facet_by]] <- factor(
      plot_data[[facet_by]],
      labels = paste0((1 - sort(unique(plot_data[[facet_by]]))) * 100, "% CI")
    )
  }

  idx_name <- plot_data$index[1]

  # Axis labels
  x_label <- switch(x_axis,
    sigma = expression(sigma ~ "(short-term SD)"),
    s     = expression(s ~ "(long-term SD)"),
    p0    = expression(p[0] ~ "(desired yield)")
  )
  y_label <- idx_name

  # Build plot
  p <- ggplot2::ggplot(plot_data,
                       ggplot2::aes(x = .data[[x_axis]], y = .data[["estimate"]])) +
    ggplot2::geom_line(color = "#2563EB", linewidth = 1) +
    ggplot2::geom_point(color = "#2563EB", size = 2)

  # Add ribbon if CI bounds exist
  has_ci <- any(!is.na(plot_data$lower) & !is.na(plot_data$upper))
  if (has_ci) {
    p <- p + ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .data[["lower"]], ymax = .data[["upper"]]),
      fill = "#93C5FD", alpha = 0.3
    )
  }

  # Reference line at 1
  p <- p + ggplot2::geom_hline(yintercept = 1, linetype = "dashed",
                                color = "#DC2626", alpha = 0.6)

  # Facet
  if (facet_by %in% names(plot_data) &&
      length(unique(plot_data[[facet_by]])) > 1) {
    p <- p + ggplot2::facet_wrap(stats::as.formula(paste("~", facet_by)))
  }

  # Theme
  p <- p +
    ggplot2::labs(
      title = paste("Sensitivity of", idx_name),
      x     = x_label,
      y     = y_label
    ) +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      plot.title       = ggplot2::element_text(face = "bold", hjust = 0.5),
      panel.grid.minor = ggplot2::element_blank()
    )

  if (interactive() || grDevices::dev.cur() > 1) {
    print(p)
  }
  invisible(p)
}

#' Compute a Process Capability Index
#'
#' Computes a single process capability or performance index from the supplied
#' parameters.
#'
#' @details
#' The twelve supported indices are:
#' \describe{
#'   \item{Cp}{\eqn{(USL - LSL) / (6\sigma)}}
#'   \item{Cpk}{\eqn{\min\bigl((USL-\mu)/(3\sigma),\;(\mu-LSL)/(3\sigma)\bigr)}}
#'   \item{Cpu}{\eqn{(USL - \mu) / (3\sigma)}}
#'   \item{Cpl}{\eqn{(\mu - LSL) / (3\sigma)}}
#'   \item{Cpm}{\eqn{(USL - LSL) / \bigl(6\sqrt{\sigma^2+(\mu-T)^2}\bigr)}}
#'   \item{Cpmk}{\eqn{\min\bigl((USL-\mu),\;(\mu-LSL)\bigr) /
#'         \bigl(3\sqrt{\sigma^2+(\mu-T)^2}\bigr)}}
#'   \item{Pp}{\eqn{(USL - LSL) / (6s)}}
#'   \item{Ppk}{\eqn{\min\bigl((USL-\bar x)/(3s),\;(\bar x-LSL)/(3s)\bigr)}}
#'   \item{Ppu}{\eqn{(USL - \bar x) / (3s)}}
#'   \item{Ppl}{\eqn{(\bar x - LSL) / (3s)}}
#'   \item{Z}{\eqn{\min\bigl((USL-\mu)/\sigma,\;(\mu-LSL)/\sigma\bigr)}}
#'   \item{Cpy}{\eqn{p / p_0 = \bigl(F(USL)-F(LSL)\bigr) /
#'         \bigl(F(UDL)-F(LDL)\bigr)}}
#' }
#'
#' @param index Character string: one of \code{"Cp"}, \code{"Cpk"},
#'   \code{"Cpu"}, \code{"Cpl"}, \code{"Cpm"}, \code{"Cpmk"},
#'   \code{"Pp"}, \code{"Ppk"}, \code{"Ppu"}, \code{"Ppl"},
#'   \code{"Z"}, or \code{"Cpy"}.
#' @param dist A \code{\link{pci_dist}} object (required for \code{"Cpy"};
#'   optional otherwise).
#' @param LSL,USL Lower and upper specification limits.
#' @param target Target value (required for \code{Cpm} and \code{Cpmk}).
#' @param mu,sigma Short-term (within-subgroup) mean and standard deviation.
#' @param xbar,s Long-term (overall) mean and standard deviation.
#' @param LDL,UDL Lower and upper desirable limits for \code{Cpy}.
#' @param p0 Desired yield for \code{Cpy}. If \code{LDL}/\code{UDL} are not
#'   supplied, they are computed from \code{p0} using
#'   \code{\link{compute_desirable_limits}}.
#'
#' @return A single numeric value, or \code{NA} with a warning when a
#'   one-sided index is requested but the relevant specification limit is
#'   \code{NULL}.
#'
#' @examples
#' pci("Cp",  LSL = 57, USL = 63, sigma = 1)            # 1.0
#' pci("Cpk", LSL = 57, USL = 63, mu = 60, sigma = 1)   # 1.0
#' pci("Z",   LSL = 57, USL = 63, mu = 60, sigma = 1)   # 3.0
#'
#' # Generalized Cpy
#' d <- pci_dist_normal(60, 1)
#' pci("Cpy", dist = d, LSL = 57, USL = 63, LDL = 57, UDL = 63) # 1.0
#'
#' @seealso \code{\link{pci_dist}}, \code{\link{pci_ci}}, \code{\link{pci_grid}}
#' @export
pci <- function(index,
                dist   = NULL,
                LSL    = NULL, USL  = NULL, target = NULL,
                mu     = NULL, sigma = NULL,
                xbar   = NULL, s     = NULL,
                LDL    = NULL, UDL   = NULL, p0 = NULL) {

  # --- validate index ---
  valid_indices <- c("Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk",
                     "Pp", "Ppk", "Ppu", "Ppl", "Z", "Cpy")
  index <- match.arg(index, valid_indices)

  # --- dispatch ---
  switch(index,
    Cp   = .calc_cp(LSL, USL, sigma),
    Cpk  = .calc_cpk(LSL, USL, mu, sigma),
    Cpu  = .calc_cpu(USL, mu, sigma),
    Cpl  = .calc_cpl(LSL, mu, sigma),
    Cpm  = .calc_cpm(LSL, USL, mu, sigma, target),
    Cpmk = .calc_cpmk(LSL, USL, mu, sigma, target),
    Pp   = .calc_pp(LSL, USL, s),
    Ppk  = .calc_ppk(LSL, USL, xbar, s),
    Ppu  = .calc_ppu(USL, xbar, s),
    Ppl  = .calc_ppl(LSL, xbar, s),
    Z    = .calc_z(LSL, USL, mu, sigma),
    Cpy  = .calc_cpy(dist, LSL, USL, LDL, UDL, p0)
  )
}

# ==================================================================
# Internal calculators
# ==================================================================

.calc_cp <- function(LSL, USL, sigma) {
  require_param(LSL, "LSL", "Cp")
  require_param(USL, "USL", "Cp")
  require_param(sigma, "sigma", "Cp")
  validate_spec_limits(LSL, USL)
  validate_positive(sigma, "sigma")
  (USL - LSL) / (6 * sigma)
}

.calc_cpk <- function(LSL, USL, mu, sigma) {
  require_param(LSL, "LSL", "Cpk")
  require_param(USL, "USL", "Cpk")
  require_param(mu, "mu", "Cpk")
  require_param(sigma, "sigma", "Cpk")
  validate_spec_limits(LSL, USL)
  validate_positive(sigma, "sigma")
  min((USL - mu) / (3 * sigma), (mu - LSL) / (3 * sigma))
}

.calc_cpu <- function(USL, mu, sigma) {
  require_param(USL, "USL", "Cpu")
  require_param(mu, "mu", "Cpu")
  require_param(sigma, "sigma", "Cpu")
  validate_positive(sigma, "sigma")
  (USL - mu) / (3 * sigma)
}

.calc_cpl <- function(LSL, mu, sigma) {
  require_param(LSL, "LSL", "Cpl")
  require_param(mu, "mu", "Cpl")
  require_param(sigma, "sigma", "Cpl")
  validate_positive(sigma, "sigma")
  (mu - LSL) / (3 * sigma)
}

.calc_cpm <- function(LSL, USL, mu, sigma, target) {
  require_param(LSL, "LSL", "Cpm")
  require_param(USL, "USL", "Cpm")
  require_param(mu, "mu", "Cpm")
  require_param(sigma, "sigma", "Cpm")
  require_param(target, "target", "Cpm")
  validate_spec_limits(LSL, USL)
  validate_positive(sigma, "sigma")
  denom <- 6 * sqrt(sigma^2 + (mu - target)^2)
  (USL - LSL) / denom
}

.calc_cpmk <- function(LSL, USL, mu, sigma, target) {
  require_param(LSL, "LSL", "Cpmk")
  require_param(USL, "USL", "Cpmk")
  require_param(mu, "mu", "Cpmk")
  require_param(sigma, "sigma", "Cpmk")
  require_param(target, "target", "Cpmk")
  validate_spec_limits(LSL, USL)
  validate_positive(sigma, "sigma")
  tau <- 3 * sqrt(sigma^2 + (mu - target)^2)
  min((USL - mu) / tau, (mu - LSL) / tau)
}

.calc_pp <- function(LSL, USL, s) {
  require_param(LSL, "LSL", "Pp")
  require_param(USL, "USL", "Pp")
  require_param(s, "s", "Pp")
  validate_spec_limits(LSL, USL)
  validate_positive(s, "s")
  (USL - LSL) / (6 * s)
}

.calc_ppk <- function(LSL, USL, xbar, s) {
  require_param(LSL, "LSL", "Ppk")
  require_param(USL, "USL", "Ppk")
  require_param(xbar, "xbar", "Ppk")
  require_param(s, "s", "Ppk")
  validate_spec_limits(LSL, USL)
  validate_positive(s, "s")
  min((USL - xbar) / (3 * s), (xbar - LSL) / (3 * s))
}

.calc_ppu <- function(USL, xbar, s) {
  require_param(USL, "USL", "Ppu")
  require_param(xbar, "xbar", "Ppu")
  require_param(s, "s", "Ppu")
  validate_positive(s, "s")
  (USL - xbar) / (3 * s)
}

.calc_ppl <- function(LSL, xbar, s) {
  require_param(LSL, "LSL", "Ppl")
  require_param(xbar, "xbar", "Ppl")
  require_param(s, "s", "Ppl")
  validate_positive(s, "s")
  (xbar - LSL) / (3 * s)
}

.calc_z <- function(LSL, USL, mu, sigma) {
  require_param(LSL, "LSL", "Z")
  require_param(USL, "USL", "Z")
  require_param(mu, "mu", "Z")
  require_param(sigma, "sigma", "Z")
  validate_spec_limits(LSL, USL)
  validate_positive(sigma, "sigma")
  min((USL - mu) / sigma, (mu - LSL) / sigma)
}

.calc_cpy <- function(dist, LSL, USL, LDL, UDL, p0) {
  if (is.null(dist) || !inherits(dist, "pci_dist")) {
    stop("Index 'Cpy' requires a 'pci_dist' object via the 'dist' argument.")
  }
  require_param(LSL, "LSL", "Cpy")
  require_param(USL, "USL", "Cpy")
  validate_spec_limits(LSL, USL)

  cdf_fn <- dist$cdf_fn

  # Compute actual yield
  p <- cdf_fn(USL) - cdf_fn(LSL)

  # Determine desirable limits
  if (!is.null(LDL) && !is.null(UDL)) {
    if (UDL <= LDL) {
      stop("UDL must be greater than LDL.")
    }
    p0_val <- cdf_fn(UDL) - cdf_fn(LDL)
  } else if (!is.null(p0)) {
    validate_p0(p0)
    limits <- compute_desirable_limits(dist, p0)
    p0_val <- p0
  } else {
    stop("Index 'Cpy' requires either 'LDL'/'UDL' or 'p0' to be specified.")
  }

  if (p0_val == 0) {
    warning("Desired yield (p0) is 0; Cpy is undefined.")
    return(NA_real_)
  }

  p / p0_val
}

# ---- Input Validation Helpers ----

#' Validate specification limits
#' @param LSL Lower specification limit
#' @param USL Upper specification limit
#' @return Invisibly returns \code{NULL}. Called for side effects (input validation).
#' @keywords internal
validate_spec_limits <- function(LSL, USL) {
  if (!is.null(LSL) && !is.null(USL)) {
    if (!is.numeric(LSL) || !is.numeric(USL)) {
      stop("LSL and USL must be numeric values.")
    }
    if (USL <= LSL) {
      stop("USL must be greater than LSL. Got USL = ", USL, ", LSL = ", LSL, ".")
    }
  }
}

#' Validate that a value is positive
#' @param x Value to check
#' @param name Name of the parameter (for error message)
#' @return Invisibly returns \code{NULL}. Called for side effects (input validation).
#' @keywords internal
validate_positive <- function(x, name) {
  if (!is.null(x)) {
    if (!is.numeric(x) || x <= 0) {
      stop("'", name, "' must be a positive number. Got ", x, ".")
    }
  }
}

#' Validate significance level alpha
#' @param alpha Significance level
#' @return Invisibly returns \code{NULL}. Called for side effects (input validation).
#' @keywords internal
validate_alpha <- function(alpha) {
  if (!is.numeric(alpha) || alpha <= 0 || alpha >= 1) {
    stop("'alpha' must be between 0 and 1 (exclusive). Got ", alpha, ".")
  }
}

#' Validate desired yield p0
#' @param p0 Desired yield
#' @return Invisibly returns \code{NULL}. Called for side effects (input validation).
#' @keywords internal
validate_p0 <- function(p0) {
  if (!is.numeric(p0) || p0 <= 0 || p0 > 1) {
    stop("'p0' must be in (0, 1]. Got ", p0, ".")
  }
}

#' Require a parameter to be non-NULL
#' @param x Value to check
#' @param name Name of the parameter
#' @param index Name of the index requiring this parameter
#' @return Invisibly returns \code{NULL}. Called for side effects (input validation).
#' @keywords internal
require_param <- function(x, name, index) {
  if (is.null(x)) {
    stop("Index '", index, "' requires '", name, "'. Please supply it.")
  }
}

# ---- Inverse-CDF Sampling ----

#' Draw random samples from a pci_dist via inverse-CDF sampling
#'
#' Generates samples by drawing U ~ Uniform(0,1) and inverting the CDF
#' using \code{\link[stats]{uniroot}}.
#'
#' @param dist A \code{pci_dist} object.
#' @param n Number of samples to draw.
#' @return Numeric vector of length \code{n}.
#' @keywords internal
inverse_cdf_sample <- function(dist, n) {
  u <- stats::runif(n)
  support <- dist$support

  # Widen support slightly for numerical safety

  lower <- if (is.finite(support[1])) support[1] else -1e6
  upper <- if (is.finite(support[2])) support[2] else 1e6

  cdf_fn <- dist$cdf_fn

  vapply(u, function(ui) {
    # Handle edge cases
    if (ui <= 0) return(lower)
    if (ui >= 1) return(upper)

    tryCatch(
      stats::uniroot(
        f = function(x) cdf_fn(x) - ui,
        lower = lower,
        upper = upper,
        tol = 1e-8
      )$root,
      error = function(e) {
        # Fallback: try expanding the interval
        tryCatch(
          stats::uniroot(
            f = function(x) cdf_fn(x) - ui,
            lower = lower - abs(lower),
            upper = upper + abs(upper),
            tol = 1e-6
          )$root,
          error = function(e2) NA_real_
        )
      }
    )
  }, numeric(1))
}

# ---- Desirable Limits for Cpy ----

#' Compute symmetric desirable limits for Cpy
#'
#' Given a \code{pci_dist} and desired yield \code{p0}, computes
#' LDL and UDL symmetric about the median of the distribution such
#' that \code{F(UDL) - F(LDL) = p0}.
#'
#' @param dist A \code{pci_dist} object.
#' @param p0 Desired yield (probability), in (0, 1].
#' @return Named numeric vector \code{c(LDL = ..., UDL = ...)}.
#' @keywords internal
compute_desirable_limits <- function(dist, p0) {
  validate_p0(p0)
  cdf_fn <- dist$cdf_fn
  support <- dist$support

  lower <- if (is.finite(support[1])) support[1] else -1e6
  upper <- if (is.finite(support[2])) support[2] else 1e6

  # Find the median
  median_val <- tryCatch(
    stats::uniroot(
      f = function(x) cdf_fn(x) - 0.5,
      lower = lower,
      upper = upper,
      tol = 1e-8
    )$root,
    error = function(e) {
      stop("Could not find the median of the distribution. ",
           "Check the CDF and support. Error: ", e$message)
    }
  )

  # If p0 == 1, use the full support
  if (p0 == 1) {
    return(c(LDL = lower, UDL = upper))
  }

  # Solve for delta such that CDF(median + delta) - CDF(median - delta) = p0
  max_delta <- max(abs(upper - median_val), abs(median_val - lower))

  delta <- tryCatch(
    stats::uniroot(
      f = function(d) cdf_fn(median_val + d) - cdf_fn(median_val - d) - p0,
      lower = 0,
      upper = max_delta,
      tol = 1e-8
    )$root,
    error = function(e) {
      stop("Could not compute desirable limits for p0 = ", p0,
           ". The desired yield may be unachievable for this distribution. ",
           "Error: ", e$message)
    }
  )

  c(LDL = median_val - delta, UDL = median_val + delta)
}

# ---- Helper to compute index from sample statistics ----

#' Compute a PCI from sample statistics
#' @return A numeric value of the calculated process capability index, or \code{NA_real_} on error/invalid input.
#' @keywords internal
compute_index_from_sample <- function(index, sample_data, LSL, USL, target,
                                       LDL, UDL, p0, dist) {
  mu_hat <- mean(sample_data)
  sigma_hat <- stats::sd(sample_data)

  if (index == "Cpy") {
    # Use empirical CDF for Cpy
    ecdf_fn <- stats::ecdf(sample_data)
    p_val <- ecdf_fn(USL) - ecdf_fn(LSL)

    if (!is.null(LDL) && !is.null(UDL)) {
      p0_val <- ecdf_fn(UDL) - ecdf_fn(LDL)
    } else if (!is.null(p0)) {
      p0_val <- p0
    } else {
      p0_val <- NA_real_
    }

    if (is.na(p0_val) || p0_val == 0) return(NA_real_)
    return(p_val / p0_val)
  }

  # For classical/performance indices, use sample stats
  pci(index = index,
      LSL = LSL, USL = USL, target = target,
      mu = mu_hat, sigma = sigma_hat,
      xbar = mu_hat, s = sigma_hat)
}

# ---- Convenience Wrappers ----
# Thin wrappers around pci() with the index pre-filled.

#' Process Potential Index (Cp)
#'
#' \eqn{C_p = (USL - LSL) / (6\sigma)}.
#'
#' @param LSL Lower specification limit.
#' @param USL Upper specification limit.
#' @param sigma Short-term (within-subgroup) standard deviation.
#' @return Numeric value of \eqn{C_p}.
#' @examples
#' cp(LSL = 57, USL = 63, sigma = 1)   # 1.0
#' @seealso \code{\link{pci}}
#' @export
cp <- function(LSL, USL, sigma) {
  pci("Cp", LSL = LSL, USL = USL, sigma = sigma)
}

#' Process Capability Index (Cpk)
#'
#' \eqn{C_{pk} = \min\bigl((USL-\mu)/(3\sigma),\;(\mu-LSL)/(3\sigma)\bigr)}.
#'
#' @param LSL Lower specification limit.
#' @param USL Upper specification limit.
#' @param mu Short-term process mean.
#' @param sigma Short-term standard deviation.
#' @return Numeric value of \eqn{C_{pk}}.
#' @examples
#' cpk(LSL = 57, USL = 63, mu = 60, sigma = 1)   # 1.0
#' @seealso \code{\link{pci}}
#' @export
cpk <- function(LSL, USL, mu, sigma) {
  pci("Cpk", LSL = LSL, USL = USL, mu = mu, sigma = sigma)
}

#' Upper Capability Index (Cpu)
#'
#' \eqn{C_{pu} = (USL - \mu) / (3\sigma)}.
#'
#' @param USL Upper specification limit.
#' @param mu Short-term process mean.
#' @param sigma Short-term standard deviation.
#' @return Numeric value of \eqn{C_{pu}}.
#' @examples
#' cpu(USL = 63, mu = 60, sigma = 1)   # 1.0
#' @seealso \code{\link{pci}}
#' @export
cpu <- function(USL, mu, sigma) {
  pci("Cpu", USL = USL, mu = mu, sigma = sigma)
}

#' Lower Capability Index (Cpl)
#'
#' \eqn{C_{pl} = (\mu - LSL) / (3\sigma)}.
#'
#' @param LSL Lower specification limit.
#' @param mu Short-term process mean.
#' @param sigma Short-term standard deviation.
#' @return Numeric value of \eqn{C_{pl}}.
#' @examples
#' cpl(LSL = 57, mu = 60, sigma = 1)   # 1.0
#' @seealso \code{\link{pci}}
#' @export
cpl <- function(LSL, mu, sigma) {
  pci("Cpl", LSL = LSL, mu = mu, sigma = sigma)
}

#' Taguchi Capability Index (Cpm)
#'
#' \eqn{C_{pm} = (USL - LSL) / \bigl(6\sqrt{\sigma^2 + (\mu - T)^2}\bigr)}.
#'
#' @param LSL Lower specification limit.
#' @param USL Upper specification limit.
#' @param mu Short-term process mean.
#' @param sigma Short-term standard deviation.
#' @param target Target value \eqn{T}.
#' @return Numeric value of \eqn{C_{pm}}.
#' @examples
#' cpm(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60)   # 1.0
#' @seealso \code{\link{pci}}
#' @export
cpm <- function(LSL, USL, mu, sigma, target) {
  pci("Cpm", LSL = LSL, USL = USL, mu = mu, sigma = sigma, target = target)
}

#' Modified Taguchi Capability Index (Cpmk)
#'
#' \eqn{C_{pmk} = \min\bigl((USL-\mu),\;(\mu-LSL)\bigr) /
#'   \bigl(3\sqrt{\sigma^2+(\mu-T)^2}\bigr)}.
#'
#' @param LSL Lower specification limit.
#' @param USL Upper specification limit.
#' @param mu Short-term process mean.
#' @param sigma Short-term standard deviation.
#' @param target Target value \eqn{T}.
#' @return Numeric value of \eqn{C_{pmk}}.
#' @examples
#' cpmk(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60)   # 1.0
#' @seealso \code{\link{pci}}
#' @export
cpmk <- function(LSL, USL, mu, sigma, target) {
  pci("Cpmk", LSL = LSL, USL = USL, mu = mu, sigma = sigma, target = target)
}

#' Performance Index (Pp)
#'
#' \eqn{P_p = (USL - LSL) / (6s)}.
#'
#' @param LSL Lower specification limit.
#' @param USL Upper specification limit.
#' @param s Long-term (overall) standard deviation.
#' @return Numeric value of \eqn{P_p}.
#' @examples
#' pp(LSL = 57, USL = 63, s = 1.5)   # 0.6667
#' @seealso \code{\link{pci}}
#' @export
pp <- function(LSL, USL, s) {
  pci("Pp", LSL = LSL, USL = USL, s = s)
}

#' Performance Capability Index (Ppk)
#'
#' \eqn{P_{pk} = \min\bigl((USL-\bar{x})/(3s),\;(\bar{x}-LSL)/(3s)\bigr)}.
#'
#' @param LSL Lower specification limit.
#' @param USL Upper specification limit.
#' @param xbar Long-term (overall) mean.
#' @param s Long-term standard deviation.
#' @return Numeric value of \eqn{P_{pk}}.
#' @examples
#' ppk(LSL = 57, USL = 63, xbar = 60, s = 1.5)   # 0.6667
#' @seealso \code{\link{pci}}
#' @export
ppk <- function(LSL, USL, xbar, s) {
  pci("Ppk", LSL = LSL, USL = USL, xbar = xbar, s = s)
}

#' Upper Performance Index (Ppu)
#'
#' \eqn{P_{pu} = (USL - \bar{x}) / (3s)}.
#'
#' @param USL Upper specification limit.
#' @param xbar Long-term mean.
#' @param s Long-term standard deviation.
#' @return Numeric value of \eqn{P_{pu}}.
#' @examples
#' ppu(USL = 63, xbar = 60, s = 1.5)   # 0.6667
#' @seealso \code{\link{pci}}
#' @export
ppu <- function(USL, xbar, s) {
  pci("Ppu", USL = USL, xbar = xbar, s = s)
}

#' Lower Performance Index (Ppl)
#'
#' \eqn{P_{pl} = (\bar{x} - LSL) / (3s)}.
#'
#' @param LSL Lower specification limit.
#' @param xbar Long-term mean.
#' @param s Long-term standard deviation.
#' @return Numeric value of \eqn{P_{pl}}.
#' @examples
#' ppl(LSL = 57, xbar = 60, s = 1.5)   # 0.6667
#' @seealso \code{\link{pci}}
#' @export
ppl <- function(LSL, xbar, s) {
  pci("Ppl", LSL = LSL, xbar = xbar, s = s)
}

#' Sigma Level (Z)
#'
#' \eqn{Z = \min\bigl((USL-\mu)/\sigma,\;(\mu-LSL)/\sigma\bigr)}.
#'
#' @param LSL Lower specification limit.
#' @param USL Upper specification limit.
#' @param mu Short-term process mean.
#' @param sigma Short-term standard deviation.
#' @return Numeric value of \eqn{Z}.
#' @examples
#' z_level(LSL = 57, USL = 63, mu = 60, sigma = 1)   # 3.0
#' @seealso \code{\link{pci}}
#' @export
z_level <- function(LSL, USL, mu, sigma) {
  pci("Z", LSL = LSL, USL = USL, mu = mu, sigma = sigma)
}

#' Generalized Process Capability Index (Cpy)
#'
#' \eqn{C_{py} = p / p_0 = \bigl(F(USL)-F(LSL)\bigr) /
#'   \bigl(F(UDL)-F(LDL)\bigr)}, where \eqn{F} is the CDF from \code{dist}
#' (Maiti, Saha & Nanda, 2010).
#'
#' @param dist A \code{\link{pci_dist}} object.
#' @param LSL Lower specification limit.
#' @param USL Upper specification limit.
#' @param LDL Lower desirable limit (optional if \code{p0} is given).
#' @param UDL Upper desirable limit (optional if \code{p0} is given).
#' @param p0 Desired yield (optional if \code{LDL}/\code{UDL} are given).
#' @return Numeric value of \eqn{C_{py}}.
#' @examples
#' d <- pci_dist_normal(mean = 60, sd = 1)
#' cpy(d, LSL = 57, USL = 63, LDL = 57, UDL = 63)   # 1.0
#' @seealso \code{\link{pci}}, \code{\link{pci_dist}}
#' @export
cpy <- function(dist, LSL, USL, LDL = NULL, UDL = NULL, p0 = NULL) {
  pci("Cpy", dist = dist, LSL = LSL, USL = USL, LDL = LDL, UDL = UDL, p0 = p0)
}

# tests/testthat/test-pci.R
# Test each classical index against hand-computed values.

test_that("Cp is computed correctly for a centered process", {
  # USL=63, LSL=57, sigma=1 -> (63-57)/(6*1) = 1.0

  expect_equal(pci("Cp", LSL = 57, USL = 63, sigma = 1), 1.0)
  # sigma=0.5 -> 6/(6*0.5) = 2.0
  expect_equal(pci("Cp", LSL = 57, USL = 63, sigma = 0.5), 2.0)
  # sigma=1.5 -> 6/9 = 0.6667
  expect_equal(pci("Cp", LSL = 57, USL = 63, sigma = 1.5), 6 / 9,
               tolerance = 1e-4)
})

test_that("Cpk is computed correctly for centered and off-center processes", {
  # Centered: mu=60, sigma=1 -> min(3/3, 3/3) = 1.0
  expect_equal(pci("Cpk", LSL = 57, USL = 63, mu = 60, sigma = 1), 1.0)
  # Off-center: mu=61 -> min(2/3, 4/3) = 0.6667
  expect_equal(pci("Cpk", LSL = 57, USL = 63, mu = 61, sigma = 1),
               2 / 3, tolerance = 1e-4)
  # Off-center the other way: mu=58 -> min(5/3, 1/3) = 0.3333
  expect_equal(pci("Cpk", LSL = 57, USL = 63, mu = 58, sigma = 1),
               1 / 3, tolerance = 1e-4)
})

test_that("Cpu is computed correctly", {
  expect_equal(pci("Cpu", USL = 63, mu = 60, sigma = 1), 1.0)
  expect_equal(pci("Cpu", USL = 63, mu = 61, sigma = 1), 2 / 3,
               tolerance = 1e-4)
})

test_that("Cpl is computed correctly", {
  expect_equal(pci("Cpl", LSL = 57, mu = 60, sigma = 1), 1.0)
  expect_equal(pci("Cpl", LSL = 57, mu = 61, sigma = 1), 4 / 3,
               tolerance = 1e-4)
})

test_that("Cpm is computed correctly", {
  # Centered on target: mu=60, target=60, sigma=1
  # -> 6 / (6*sqrt(1+0)) = 1.0

  expect_equal(pci("Cpm", LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60),
               1.0)
  # Off-target: mu=60, target=59, sigma=1
  # -> 6 / (6*sqrt(1 + 1)) = 6/(6*sqrt(2)) = 1/sqrt(2) ~ 0.7071
  expect_equal(pci("Cpm", LSL = 57, USL = 63, mu = 60, sigma = 1, target = 59),
               1 / sqrt(2), tolerance = 1e-4)
})

test_that("Cpmk is computed correctly", {
  # Centered on target: -> 1.0
  expect_equal(pci("Cpmk", LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60),
               1.0)
  # Off-center, off-target: mu=61, target=59, sigma=1
  # tau = 3*sqrt(1 + 4) = 3*sqrt(5)
  # min((63-61)/tau, (61-57)/tau) = min(2, 4) / (3*sqrt(5)) = 2/(3*sqrt(5))
  expect_equal(pci("Cpmk", LSL = 57, USL = 63, mu = 61, sigma = 1, target = 59),
               2 / (3 * sqrt(5)), tolerance = 1e-4)
})

test_that("Pp is computed correctly", {
  # s=1.5 -> 6/(6*1.5) = 0.6667
  expect_equal(pci("Pp", LSL = 57, USL = 63, s = 1.5), 6 / 9,
               tolerance = 1e-4)
})

test_that("Ppk is computed correctly", {
  # Centered: xbar=60, s=1.5 -> min(3/4.5, 3/4.5) = 0.6667
  expect_equal(pci("Ppk", LSL = 57, USL = 63, xbar = 60, s = 1.5),
               3 / 4.5, tolerance = 1e-4)
})

test_that("Ppu is computed correctly", {
  expect_equal(pci("Ppu", USL = 63, xbar = 60, s = 1.5), 3 / 4.5,
               tolerance = 1e-4)
})

test_that("Ppl is computed correctly", {
  expect_equal(pci("Ppl", LSL = 57, xbar = 60, s = 1.5), 3 / 4.5,
               tolerance = 1e-4)
})

test_that("Z (sigma level) is computed correctly", {
  # Centered: min(3/1, 3/1) = 3.0
  expect_equal(pci("Z", LSL = 57, USL = 63, mu = 60, sigma = 1), 3.0)
  # Off-center: mu=61 -> min(2, 4) = 2.0
  expect_equal(pci("Z", LSL = 57, USL = 63, mu = 61, sigma = 1), 2.0)
})

test_that("convenience wrappers return the same values as pci()", {
  expect_equal(cp(LSL = 57, USL = 63, sigma = 1),
               pci("Cp", LSL = 57, USL = 63, sigma = 1))
  expect_equal(cpk(LSL = 57, USL = 63, mu = 60, sigma = 1),
               pci("Cpk", LSL = 57, USL = 63, mu = 60, sigma = 1))
  expect_equal(cpu(USL = 63, mu = 60, sigma = 1),
               pci("Cpu", USL = 63, mu = 60, sigma = 1))
  expect_equal(cpl(LSL = 57, mu = 60, sigma = 1),
               pci("Cpl", LSL = 57, mu = 60, sigma = 1))
  expect_equal(cpm(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60),
               pci("Cpm", LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60))
  expect_equal(cpmk(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60),
               pci("Cpmk", LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60))
  expect_equal(pp(LSL = 57, USL = 63, s = 1.5),
               pci("Pp", LSL = 57, USL = 63, s = 1.5))
  expect_equal(ppk(LSL = 57, USL = 63, xbar = 60, s = 1.5),
               pci("Ppk", LSL = 57, USL = 63, xbar = 60, s = 1.5))
  expect_equal(ppu(USL = 63, xbar = 60, s = 1.5),
               pci("Ppu", USL = 63, xbar = 60, s = 1.5))
  expect_equal(ppl(LSL = 57, xbar = 60, s = 1.5),
               pci("Ppl", LSL = 57, xbar = 60, s = 1.5))
  expect_equal(z_level(LSL = 57, USL = 63, mu = 60, sigma = 1),
               pci("Z", LSL = 57, USL = 63, mu = 60, sigma = 1))
})

#' Create a Distribution Specification for Process Capability Analysis
#'
#' Constructs an S3 object of class \code{"pci_dist"} that encapsulates the
#' probability density function (PDF) and cumulative distribution function (CDF)
#' of a quality characteristic. This object is passed to \code{\link{pci}},
#' \code{\link{pci_ci}}, and \code{\link{pci_grid}} when computing generalised
#' indices such as \eqn{C_{py}}.
#'
#' @param pdf A function \code{function(x, ...)} returning the density at
#'   \code{x}. Additional distribution parameters are passed via \code{params}.
#' @param cdf A function \code{function(x, ...)} returning the cumulative
#'   probability \eqn{P(Z \le x)}. If \code{NULL}, a numerical CDF is derived
#'   via \code{\link[stats]{integrate}} over \code{pdf}.
#' @param params A named list of distribution parameters forwarded to
#'   \code{pdf} and \code{cdf} via \code{\link{do.call}}.
#' @param support A length-2 numeric vector giving the lower and upper bounds
#'   of the distribution's support (default \code{c(-Inf, Inf)}).
#'
#' @return An S3 object of class \code{"pci_dist"} with elements:
#'   \item{pdf_fn}{Closure that evaluates the PDF at a given point.}
#'   \item{cdf_fn}{Closure that evaluates the CDF at a given point.}
#'   \item{params}{The supplied parameter list.}
#'   \item{support}{The supplied support bounds.}
#'   \item{cdf_is_numeric}{Logical; \code{TRUE} if CDF was derived numerically.}
#'
#' @examples
#' # Normal distribution
#' d <- pci_dist(
#'   pdf = function(x, mean, sd) dnorm(x, mean, sd),
#'   cdf = function(x, mean, sd) pnorm(x, mean, sd),
#'   params = list(mean = 0, sd = 1)
#' )
#' d$cdf_fn(0)   # 0.5
#'
#' # Weibull with only PDF (CDF derived numerically)
#' d2 <- pci_dist(
#'   pdf = function(x, shape, scale) dweibull(x, shape, scale),
#'   params = list(shape = 2, scale = 10),
#'   support = c(0, 60)
#' )
#'
#' @export
pci_dist <- function(pdf, cdf = NULL, params = list(), support = c(-Inf, Inf)) {


  # --- validate ---

  if (!is.function(pdf)) {
    stop("'pdf' must be a function.")
  }
  if (!is.null(cdf) && !is.function(cdf)) {
    stop("'cdf' must be a function or NULL.")
  }
  if (!is.list(params)) {
    stop("'params' must be a named list.")
  }
  if (!is.numeric(support) || length(support) != 2) {
    stop("'support' must be a numeric vector of length 2.")
  }
  if (support[2] <= support[1]) {
    stop("support[2] must be greater than support[1].")
  }

  # --- build closures ---
  pdf_fn <- function(x) {
    do.call(pdf, c(list(x), params))
  }


  if (!is.null(cdf)) {
    cdf_fn <- function(x) {
      do.call(cdf, c(list(x), params))
    }
    cdf_is_numeric <- FALSE
  } else {
    # Derive CDF numerically via integrate
    low <- if (is.finite(support[1])) support[1] else -Inf
    cdf_fn <- function(x) {
      if (length(x) > 1) return(vapply(x, cdf_fn, numeric(1)))
      if (is.finite(support[1]) && x <= support[1]) return(0)
      if (is.finite(support[2]) && x >= support[2]) return(1)
      tryCatch(
        stats::integrate(pdf_fn, lower = low, upper = x,
                         rel.tol = 1e-8, abs.tol = 1e-12)$value,
        error = function(e) {
          warning("Numerical integration failed at x = ", x,
                  ". Returning NA. Error: ", e$message)
          NA_real_
        }
      )
    }
    cdf_is_numeric <- TRUE
  }

  structure(
    list(
      pdf_fn         = pdf_fn,
      cdf_fn         = cdf_fn,
      params         = params,
      support        = support,
      cdf_is_numeric = cdf_is_numeric
    ),
    class = "pci_dist"
  )
}

#' Print method for pci_dist objects
#'
#' @param x A \code{pci_dist} object.
#' @param ... Additional arguments (ignored).
#'
#' @return Invisibly returns the input \code{x} of class \code{"pci_dist"}.
#'   Called for side effect of printing distribution parameters and specification details to the console.
#' @export
print.pci_dist <- function(x, ...) {
  cat("Process Capability Distribution Specification\n")
  cat("  Support : [", x$support[1], ", ", x$support[2], "]\n", sep = "")
  if (length(x$params) > 0) {
    pstr <- paste(names(x$params), unlist(x$params), sep = " = ", collapse = ", ")
    cat("  Parameters: ", pstr, "\n", sep = "")
  }
  if (x$cdf_is_numeric) {
    cat("  CDF: derived numerically via integrate()\n")
  } else {
    cat("  CDF: user-supplied (closed-form)\n")
  }
  invisible(x)
}

# ------------------------------------------------------------------
#' Normal Distribution Shortcut
#'
#' Convenience constructor for a \code{\link{pci_dist}} based on the normal
#' distribution. Useful for sanity-checking that \eqn{C_{py}} collapses to
#' the classical indices when \eqn{F} is the normal CDF.
#'
#' @param mean Mean of the normal distribution.
#' @param sd Standard deviation of the normal distribution.
#' @return An S3 object of class \code{"pci_dist"} encapsulating the normal
#'   distribution PDF and CDF functions, parameter list, and support bounds.
#'
#' @examples
#' d <- pci_dist_normal(mean = 60, sd = 1)
#' d$cdf_fn(60)   # 0.5
#' d$pdf_fn(60)   # ~ 0.3989
#'
#' @export
pci_dist_normal <- function(mean = 0, sd = 1) {
  if (!is.numeric(mean) || length(mean) != 1) stop("'mean' must be a single number.")
  if (!is.numeric(sd) || length(sd) != 1 || sd <= 0) stop("'sd' must be a single positive number.")

  pci_dist(
    pdf    = function(x, mean, sd) stats::dnorm(x, mean = mean, sd = sd),
    cdf    = function(x, mean, sd) stats::pnorm(x, mean = mean, sd = sd),
    params = list(mean = mean, sd = sd),
    support = c(mean - 8 * sd, mean + 8 * sd)
  )
}

# tests/testthat/test-cpy.R
# Test the generalized Cpy index.

test_that("Cpy equals 1.0 under normal when spec limits equal desirable limits", {
  d <- pci_dist_normal(mean = 60, sd = 1)

  # LDL = mu - 3*sigma = 57, UDL = mu + 3*sigma = 63
  # Both actual yield and desired yield are pnorm(63,60,1) - pnorm(57,60,1)
  result <- pci("Cpy", dist = d, LSL = 57, USL = 63, LDL = 57, UDL = 63)
  expect_equal(result, 1.0, tolerance = 1e-6)
})

test_that("Cpy > 1 when spec limits are wider than desirable limits", {
  d <- pci_dist_normal(mean = 60, sd = 1)

  # Wider spec: LSL=56, USL=64 vs desirable LDL=57, UDL=63
  result <- pci("Cpy", dist = d, LSL = 56, USL = 64, LDL = 57, UDL = 63)
  expect_true(result > 1.0)
})

test_that("Cpy < 1 when spec limits are narrower than desirable limits", {
  d <- pci_dist_normal(mean = 60, sd = 1)

  # Narrower spec: LSL=58, USL=62 vs desirable LDL=57, UDL=63
  result <- pci("Cpy", dist = d, LSL = 58, USL = 62, LDL = 57, UDL = 63)
  expect_true(result < 1.0)
})

test_that("Cpy with p0 auto-computes desirable limits correctly", {
  d <- pci_dist_normal(mean = 60, sd = 1)

  # p0 = pnorm(63,60,1) - pnorm(57,60,1) ~ 0.9973002
  p0_val <- stats::pnorm(63, 60, 1) - stats::pnorm(57, 60, 1)

  result <- pci("Cpy", dist = d, LSL = 57, USL = 63, p0 = p0_val)
  expect_equal(result, 1.0, tolerance = 0.01)
})

test_that("Cpy works with non-normal distribution (Weibull)", {
  d_weibull <- pci_dist(
    pdf    = function(x, shape, scale) stats::dweibull(x, shape = shape, scale = scale),
    cdf    = function(x, shape, scale) stats::pweibull(x, shape = shape, scale = scale),
    params = list(shape = 2, scale = 10),
    support = c(0, 50)
  )

  result <- pci("Cpy", dist = d_weibull, LSL = 2, USL = 20, p0 = 0.95)
  expect_true(is.numeric(result))
  expect_true(!is.na(result))
  expect_true(result > 0)
})

test_that("Cpy works when only pdf is supplied (CDF derived numerically)", {
  d_pdf_only <- pci_dist(
    pdf     = function(x, mean, sd) stats::dnorm(x, mean = mean, sd = sd),
    cdf     = NULL,
    params  = list(mean = 60, sd = 1),
    support = c(52, 68)
  )

  result <- pci("Cpy", dist = d_pdf_only, LSL = 57, USL = 63,
                LDL = 57, UDL = 63)
  expect_equal(result, 1.0, tolerance = 0.01)
})

test_that("cpy convenience wrapper works", {
  d <- pci_dist_normal(mean = 60, sd = 1)
  r1 <- cpy(d, LSL = 57, USL = 63, LDL = 57, UDL = 63)
  r2 <- pci("Cpy", dist = d, LSL = 57, USL = 63, LDL = 57, UDL = 63)
  expect_equal(r1, r2)
})

# tests/testthat/test-validation.R
# Test that input validation produces clear errors.

test_that("USL <= LSL produces an error", {
  expect_error(pci("Cp", LSL = 63, USL = 57, sigma = 1),
               "USL must be greater than LSL")
  expect_error(pci("Cp", LSL = 60, USL = 60, sigma = 1),
               "USL must be greater than LSL")
})

test_that("negative or zero sigma produces an error", {
  expect_error(pci("Cp", LSL = 57, USL = 63, sigma = -1),
               "positive")
  expect_error(pci("Cp", LSL = 57, USL = 63, sigma = 0),
               "positive")
})

test_that("negative or zero s produces an error", {
  expect_error(pci("Pp", LSL = 57, USL = 63, s = -1),
               "positive")
  expect_error(pci("Pp", LSL = 57, USL = 63, s = 0),
               "positive")
})

test_that("invalid alpha produces an error in pci_ci", {
  d <- pci_dist_normal(mean = 60, sd = 1)
  expect_error(pci_ci("Cp", dist = d, n = 30, LSL = 57, USL = 63, alpha = 0),
               "alpha.*between 0 and 1")
  expect_error(pci_ci("Cp", dist = d, n = 30, LSL = 57, USL = 63, alpha = 1),
               "alpha.*between 0 and 1")
  expect_error(pci_ci("Cp", dist = d, n = 30, LSL = 57, USL = 63, alpha = -0.5),
               "alpha.*between 0 and 1")
  expect_error(pci_ci("Cp", dist = d, n = 30, LSL = 57, USL = 63, alpha = 1.5),
               "alpha.*between 0 and 1")
})

test_that("Cpy without dist produces an error", {
  expect_error(pci("Cpy", LSL = 57, USL = 63, p0 = 0.95),
               "pci_dist")
})

test_that("Cpy without p0 or LDL/UDL produces an error", {
  d <- pci_dist_normal(mean = 60, sd = 1)
  expect_error(pci("Cpy", dist = d, LSL = 57, USL = 63),
               "LDL.*UDL.*p0")
})

test_that("invalid index name produces an error", {
  expect_error(pci("InvalidIndex"),
               "'arg' should be one of")
})

test_that("pci_dist with non-function pdf produces an error", {
  expect_error(pci_dist(pdf = "not_a_function"),
               "function")
})

test_that("missing required parameters produce errors", {
  expect_error(pci("Cp", LSL = 57, USL = 63),
               "sigma")
  expect_error(pci("Cpk", LSL = 57, USL = 63, sigma = 1),
               "mu")
  expect_error(pci("Cpm", LSL = 57, USL = 63, mu = 60, sigma = 1),
               "target")
  expect_error(pci("Ppk", LSL = 57, USL = 63, s = 1.5),
               "xbar")
})

test_that("pci_dist_normal with invalid sd produces an error", {
  expect_error(pci_dist_normal(mean = 0, sd = -1), "positive")
  expect_error(pci_dist_normal(mean = 0, sd = 0),  "positive")
})

test_that("pci_dist with bad support produces an error", {
  expect_error(
    pci_dist(pdf = dnorm, support = c(10, 5)),
    "support"
  )
})

#' Bootstrap Confidence Interval for a Process Capability Index
#'
#' Computes a confidence interval for a PCI via parametric or nonparametric
#' bootstrap.
#'
#' @details
#' **Parametric bootstrap** (when \code{data = NULL}): \code{B} samples of size
#' \code{n} are drawn from \code{dist} via inverse-CDF sampling.
#' For each sample the process mean, standard deviation (and, for
#' \eqn{C_{py}}, the empirical CDF) are estimated, and the index is
#' recomputed.
#'
#' **Nonparametric bootstrap** (when \code{data} is supplied): \code{B}
#' resamples of the same length are drawn with replacement from \code{data}.
#'
#' Two interval methods are available:
#' \describe{
#'   \item{percentile}{Uses the \code{alpha/2} and \code{1 - alpha/2}
#'     quantiles of the bootstrap distribution.}
#'   \item{bca}{Bias-corrected and accelerated interval using jackknife
#'     acceleration and bootstrap bias correction (Efron, 1987).}
#' }
#'
#' @param index Character: one of the 12 index names (see \code{\link{pci}}).
#' @param dist A \code{\link{pci_dist}} object.
#' @param n Integer sample size.
#' @param data Optional numeric vector of observed data. If supplied,
#'   nonparametric bootstrap is used instead of parametric.
#' @param LSL,USL Specification limits.
#' @param target Target value (for \code{Cpm}/\code{Cpmk}).
#' @param LDL,UDL Desirable limits (for \code{Cpy}).
#' @param p0 Desired yield (for \code{Cpy}).
#' @param alpha Significance level; the interval covers \eqn{1 - \alpha}.
#'   Default 0.05.
#' @param B Number of bootstrap replications. Default 2000.
#' @param method Character: \code{"percentile"} (default) or \code{"bca"}.
#'
#' @return An S3 object of class \code{"pci_ci"} with components:
#'   \item{index}{The index name.}
#'   \item{estimate}{Point estimate.}
#'   \item{lower}{Lower CI bound.}
#'   \item{upper}{Upper CI bound.}
#'   \item{alpha}{The significance level used.}
#'   \item{method}{The interval method used.}
#'   \item{boot_values}{Numeric vector of all \code{B} bootstrap estimates.}
#'
#' @examples
#' d <- pci_dist_normal(mean = 60, sd = 1)
#' ci <- pci_ci("Cp", dist = d, n = 30, LSL = 57, USL = 63,
#'              alpha = 0.05, B = 500)
#' print(ci)
#'
#' @seealso \code{\link{pci}}, \code{\link{pci_grid}}
#' @export
pci_ci <- function(index, dist, n,
                   data    = NULL,
                   LSL     = NULL, USL    = NULL, target = NULL,
                   LDL     = NULL, UDL    = NULL, p0     = NULL,
                   alpha   = 0.05,
                   B       = 2000,
                   method  = c("percentile", "bca")) {

  method <- match.arg(method)
  validate_alpha(alpha)

  valid_indices <- c("Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk",
                     "Pp", "Ppk", "Ppu", "Ppl", "Z", "Cpy")
  index <- match.arg(index, valid_indices)

  if (!is.numeric(B) || B < 1) stop("'B' must be a positive integer.")
  B <- as.integer(B)

  # ---- helper: compute index from a sample vector -----------------------
  idx_from_sample <- function(samp) {
    mu_hat    <- mean(samp)
    sigma_hat <- stats::sd(samp)

    if (index == "Cpy") {
      ecdf_fn <- stats::ecdf(samp)
      p_val   <- ecdf_fn(USL) - ecdf_fn(LSL)

      if (!is.null(LDL) && !is.null(UDL)) {
        p0_val <- ecdf_fn(UDL) - ecdf_fn(LDL)
      } else if (!is.null(p0)) {
        p0_val <- p0
      } else {
        return(NA_real_)
      }
      if (is.na(p0_val) || p0_val == 0) return(NA_real_)
      return(p_val / p0_val)
    }

    # For classical/performance indices use sample stats
    tryCatch(
      pci(index  = index,
          LSL    = LSL,  USL    = USL,  target = target,
          mu     = mu_hat, sigma = sigma_hat,
          xbar   = mu_hat, s     = sigma_hat),
      error = function(e) NA_real_
    )
  }

  # ---- point estimate ---------------------------------------------------
  if (!is.null(data)) {
    data <- as.numeric(data)
    n    <- length(data)
    estimate <- idx_from_sample(data)
  } else {
    # parametric: draw one "reference" sample for the estimate
    if (missing(dist) || is.null(dist)) {
      stop("Either 'data' or 'dist' must be supplied.")
    }
    if (missing(n) || is.null(n)) {
      stop("'n' (sample size) must be supplied for parametric bootstrap.")
    }
    ref_sample <- inverse_cdf_sample(dist, n)
    estimate   <- idx_from_sample(ref_sample)
  }

  # ---- bootstrap --------------------------------------------------------
  boot_values <- numeric(B)

  if (!is.null(data)) {
    # nonparametric bootstrap
    for (b in seq_len(B)) {
      samp <- sample(data, size = n, replace = TRUE)
      boot_values[b] <- idx_from_sample(samp)
    }
  } else {
    # parametric bootstrap
    for (b in seq_len(B)) {
      samp <- inverse_cdf_sample(dist, n)
      boot_values[b] <- idx_from_sample(samp)
    }
  }

  # Remove NAs
  boot_ok <- boot_values[!is.na(boot_values)]
  if (length(boot_ok) < 10) {
    warning("Fewer than 10 valid bootstrap replicates. CI may be unreliable.")
  }

  # ---- confidence limits ------------------------------------------------
  if (method == "percentile") {
    lower <- as.numeric(stats::quantile(boot_ok, probs = alpha / 2))
    upper <- as.numeric(stats::quantile(boot_ok, probs = 1 - alpha / 2))

  } else {
    # BCa method
    # bias correction factor
    z0 <- stats::qnorm(mean(boot_ok < estimate))

    # acceleration via jackknife
    if (!is.null(data)) {
      jack_data <- data
    } else {
      jack_data <- ref_sample
    }

    nn <- length(jack_data)
    jack_vals <- numeric(nn)
    for (i in seq_len(nn)) {
      jack_vals[i] <- idx_from_sample(jack_data[-i])
    }
    theta_dot <- mean(jack_vals, na.rm = TRUE)
    diffs     <- theta_dot - jack_vals
    a_num     <- sum(diffs^3, na.rm = TRUE)
    a_den     <- 6 * (sum(diffs^2, na.rm = TRUE))^1.5
    acc       <- if (a_den == 0) 0 else a_num / a_den

    # adjusted quantiles
    z_lo <- stats::qnorm(alpha / 2)
    z_hi <- stats::qnorm(1 - alpha / 2)

    alpha1 <- stats::pnorm(z0 + (z0 + z_lo) / (1 - acc * (z0 + z_lo)))
    alpha2 <- stats::pnorm(z0 + (z0 + z_hi) / (1 - acc * (z0 + z_hi)))

    # Clamp to [0, 1]
    alpha1 <- max(min(alpha1, 1), 0)
    alpha2 <- max(min(alpha2, 1), 0)

    lower <- as.numeric(stats::quantile(boot_ok, probs = alpha1))
    upper <- as.numeric(stats::quantile(boot_ok, probs = alpha2))
  }

  structure(
    list(
      index       = index,
      estimate    = estimate,
      lower       = lower,
      upper       = upper,
      alpha       = alpha,
      method      = method,
      boot_values = boot_values
    ),
    class = "pci_ci"
  )
}


#' Print method for pci_ci objects
#'
#' @param x A \code{pci_ci} object.
#' @param ... Additional arguments (ignored).
#'
#' @return Invisibly returns the input \code{x} of class \code{"pci_ci"}.
#'   Called for side effect of printing formatted confidence interval details to the console.
#' @export
print.pci_ci <- function(x, ...) {
  pct <- round((1 - x$alpha) * 100, 1)
  cat(sprintf("  Index    : %s\n", x$index))
  cat(sprintf("  Estimate : %.4f\n", x$estimate))
  cat(sprintf("  %g%% CI [%s]: (%.4f, %.4f)\n",
              pct, x$method, x$lower, x$upper))
  invisible(x)
}

---
title: "Getting Started with ProcessCapabilityR"
author: "Shikhar Tyagi, Sumit Kumar, Vrijesh Tripathi"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with ProcessCapabilityR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(ProcessCapabilityR)
```

## Introduction

Process Capability Indices (PCIs) are fundamental tools in Statistical Quality
Control. They quantify the ability of a manufacturing or service process to
produce output within engineering specification limits. Classical indices such
as $C_p$ and $C_{pk}$ assume the quality characteristic follows a normal
distribution.

**ProcessCapabilityR** extends this framework by implementing the
*Generalized Process Capability Index* $C_{py}$ (Maiti, Saha & Nanda, 2010),
which works for **any** continuous or discrete distribution the user plugs in.
The classical indices are recovered as special cases under normality.

This vignette demonstrates the full API through worked examples.

---

## 1. Normal-Case Sanity Check

Consider a centered process with $USL = 63$, $LSL = 57$, $\mu = 60$,
$\sigma = 1$.

```{r classical-centered}
# All classical capability indices
cat("Cp  =", cp(LSL = 57, USL = 63, sigma = 1), "\n")
cat("Cpk =", cpk(LSL = 57, USL = 63, mu = 60, sigma = 1), "\n")
cat("Cpu =", cpu(USL = 63, mu = 60, sigma = 1), "\n")
cat("Cpl =", cpl(LSL = 57, mu = 60, sigma = 1), "\n")
cat("Z   =", z_level(LSL = 57, USL = 63, mu = 60, sigma = 1), "\n")
```

For a perfectly centered process the specification width equals $6\sigma$,
so $C_p = C_{pk} = 1.0$ and $Z = 3$.

```{r taguchi-centered}
# Taguchi indices with target = process mean
cat("Cpm  =", cpm(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60), "\n")
cat("Cpmk =", cpmk(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60), "\n")
```

---

## 2. Off-Center Process

Now shift the mean to $\mu = 61$ (off-center but within limits):

```{r off-center}
cat("Cp  =", cp(LSL = 57, USL = 63, sigma = 1), "\n")
cat("Cpk =", cpk(LSL = 57, USL = 63, mu = 61, sigma = 1), "\n")
cat("Cpu =", cpu(USL = 63, mu = 61, sigma = 1), "\n")
cat("Cpl =", cpl(LSL = 57, mu = 61, sigma = 1), "\n")
```

Note that $C_p$ is unchanged (it ignores centering), while $C_{pk}$ drops
to 0.667 because the process is closer to the USL.

---

## 3. Taguchi Indices — Off-Target Example

Set $\mu = 60$, $\sigma = 1$, and a target $T = 59$ (target is different
from the process mean):

```{r taguchi-off}
cat("Cpm  =", cpm(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 59), "\n")
cat("Cpmk =", cpmk(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 59), "\n")
```

$C_{pm} = 1/\sqrt{2} \approx 0.707$ — the Taguchi denominator
$\sqrt{\sigma^2 + (\mu - T)^2}$ inflates because $\mu \neq T$.

---

## 4. Performance Indices

Using $\bar{x} = 60$, $s = 1.5$ (long-term SD, larger than $\sigma$):

```{r performance}
cat("Pp  =", pp(LSL = 57, USL = 63, s = 1.5), "\n")
cat("Ppk =", ppk(LSL = 57, USL = 63, xbar = 60, s = 1.5), "\n")
cat("Ppu =", ppu(USL = 63, xbar = 60, s = 1.5), "\n")
cat("Ppl =", ppl(LSL = 57, xbar = 60, s = 1.5), "\n")
```

With $s > \sigma$, the performance indices ($P_p = 0.667$) are lower
than the capability indices ($C_p = 1.0$), indicating extra variation
from long-term sources.

---

## 5. Generalized $C_{py}$: Normal Distribution

$C_{py}$ is defined as the ratio of actual to desired yield:
$$C_{py} = \frac{F(USL) - F(LSL)}{F(UDL) - F(LDL)} = \frac{p}{p_0}$$

Under normality with $LDL = \mu - 3\sigma$ and $UDL = \mu + 3\sigma$,
the actual and desired yields are identical, so $C_{py} = 1$:

```{r cpy-normal}
d_norm <- pci_dist_normal(mean = 60, sd = 1)

cat("Cpy (spec = desirable) =",
    cpy(d_norm, LSL = 57, USL = 63, LDL = 57, UDL = 63), "\n")
```

With wider spec limits ($LSL = 56$, $USL = 64$), $C_{py} > 1$:

```{r cpy-wider}
cat("Cpy (wider spec) =",
    cpy(d_norm, LSL = 56, USL = 64, LDL = 57, UDL = 63), "\n")
```

---

## 6. Non-Normal Example: Weibull Distribution

```{r cpy-weibull}
d_weibull <- pci_dist(
  pdf    = function(x, shape, scale) dweibull(x, shape = shape, scale = scale),
  cdf    = function(x, shape, scale) pweibull(x, shape = shape, scale = scale),
  params = list(shape = 2, scale = 10),
  support = c(0, 50)
)

cat("Cpy (Weibull, p0=0.95) =",
    cpy(d_weibull, LSL = 2, USL = 20, p0 = 0.95), "\n")
cat("Cpy (Weibull, p0=0.90) =",
    cpy(d_weibull, LSL = 2, USL = 20, p0 = 0.90), "\n")
```

---

## 7. Non-Normal Example: Gamma Distribution

```{r cpy-gamma}
d_gamma <- pci_dist(
  pdf    = function(x, shape, rate) dgamma(x, shape = shape, rate = rate),
  cdf    = function(x, shape, rate) pgamma(x, shape = shape, rate = rate),
  params = list(shape = 5, rate = 0.5),
  support = c(0, 60)
)

cat("Cpy (Gamma, p0=0.95) =",
    cpy(d_gamma, LSL = 2, USL = 25, p0 = 0.95), "\n")
```

---

## 8. Bootstrap Confidence Intervals

```{r ci-example}
set.seed(42)
d <- pci_dist_normal(mean = 60, sd = 1)

ci_pct <- pci_ci("Cp", dist = d, n = 30, LSL = 57, USL = 63,
                 alpha = 0.05, B = 500, method = "percentile")
print(ci_pct)
```

```{r ci-bca}
set.seed(42)
ci_bca <- pci_ci("Cp", dist = d, n = 30, LSL = 57, USL = 63,
                 alpha = 0.05, B = 500, method = "bca")
print(ci_bca)
```

---

## 9. Sensitivity Sweep: $\sigma$

Sweep $\sigma$ from 0.5 to 2.0 with confidence bands at
90%, 95%, 97%, and 99%:

```{r grid-sigma, eval = FALSE}
d <- pci_dist_normal(mean = 60, sd = 1)
grid_sigma <- pci_grid("Cp",
                       dist = d,
                       LSL = 57, USL = 63,
                       sigma_vals = seq(0.5, 2.0, by = 0.1),
                       mu = 60,
                       alpha_vals = c(0.10, 0.05, 0.03, 0.01),
                       n = 30, B = 500)
head(grid_sigma)
```

```{r grid-sigma-plot, eval = FALSE}
plot(grid_sigma, x_axis = "sigma")
```

---

## 10. Sensitivity Sweep: $p_0$ for $C_{py}$

```{r grid-cpy, eval = FALSE}
d <- pci_dist_normal(mean = 60, sd = 1)
grid_cpy <- pci_grid("Cpy",
                     dist = d,
                     LSL = 57, USL = 63,
                     p0_vals = c(0.90, 0.95, 0.99),
                     alpha_vals = c(0.10, 0.05, 0.03, 0.01),
                     n = 30, B = 500)
head(grid_cpy)
```

```{r grid-cpy-plot, eval = FALSE}
plot(grid_cpy, x_axis = "p0")
```

---

## References

- Kane, V.E. (1986). Process capability indices. *Journal of Quality
  Technology*, 18(1), 41–52.
- Chan, L.K., Cheng, S.W., & Spiring, F.A. (1988). A new measure of
  process capability: $C_{pm}$. *Journal of Quality Technology*, 20(3),
  162–175.
- Pearn, W.L., Kotz, S., & Johnson, N.L. (1992). Distributional and
  inferential properties of process capability indices. *Journal of Quality
  Technology*, 24(4), 216–231.
- Kotz, S., & Johnson, N.L. (2002). Process capability indices — a review,
  1992–2000. *Journal of Quality Technology*, 34(1), 2–19.
- Maiti, S.S., Saha, M., & Nanda, A.K. (2010). On generalizing process
  capability indices. *Quality Technology & Quantitative Management*,
  7(3), 279–300.
- Montgomery, D.C. (2020). *Introduction to Statistical Quality Control*
  (8th ed.). Wiley.
- Harry, M., & Schroeder, R. (2000). *Six Sigma*. Doubleday.
- AIAG (2005). *Statistical Process Control (SPC) Reference Manual*
  (2nd ed.).
- Juran, J.M. (1974). *Quality Control Handbook* (3rd ed.). McGraw-Hill.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{validate_positive}
\alias{validate_positive}
\title{Validate that a value is positive}
\usage{
validate_positive(x, name)
}
\arguments{
\item{x}{Value to check}

\item{name}{Name of the parameter (for error message)}
}
\value{
Invisibly returns \code{NULL}. Called for side effects (input validation).
}
\description{
Validate that a value is positive
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{require_param}
\alias{require_param}
\title{Require a parameter to be non-NULL}
\usage{
require_param(x, name, index)
}
\arguments{
\item{x}{Value to check}

\item{name}{Name of the parameter}

\item{index}{Name of the index requiring this parameter}
}
\value{
Invisibly returns \code{NULL}. Called for side effects (input validation).
}
\description{
Require a parameter to be non-NULL
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_pci_grid.R
\name{plot.pci_grid}
\alias{plot.pci_grid}
\title{Plot a Process Capability Sensitivity Grid}
\usage{
\method{plot}{pci_grid}(x, x_axis = c("sigma", "s", "p0"), facet_by = "alpha", ...)
}
\arguments{
\item{x}{A \code{\link{pci_grid}} object.}

\item{x_axis}{Character: which sweep variable to place on the x-axis.
One of \code{"sigma"}, \code{"s"}, or \code{"p0"}.}

\item{facet_by}{Character: column name to facet by. Default \code{"alpha"}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns a \code{\link[ggplot2]{ggplot}} object representing
  the sensitivity plot. Called primarily for its side effect of rendering a
  line-and-ribbon plot to the active graphic device.
}
\description{
Produces a \pkg{ggplot2} line-and-ribbon plot showing the PCI estimate
and confidence bands as a function of the chosen sweep variable, faceted
by significance level.
}
\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  grid <- pci_grid("Cp", LSL = 57, USL = 63,
                   sigma_vals = seq(0.5, 2, 0.1), mu = 60,
                   alpha_vals = c(0.05, 0.01),
                   n = 30, B = 200,
                   dist = pci_dist_normal(60, 1))
  plot(grid, x_axis = "sigma")
}
}

}
\seealso{
\code{\link{pci_grid}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pci.R
\name{pci}
\alias{pci}
\title{Compute a Process Capability Index}
\usage{
pci(
  index,
  dist = NULL,
  LSL = NULL,
  USL = NULL,
  target = NULL,
  mu = NULL,
  sigma = NULL,
  xbar = NULL,
  s = NULL,
  LDL = NULL,
  UDL = NULL,
  p0 = NULL
)
}
\arguments{
\item{index}{Character string: one of \code{"Cp"}, \code{"Cpk"},
\code{"Cpu"}, \code{"Cpl"}, \code{"Cpm"}, \code{"Cpmk"},
\code{"Pp"}, \code{"Ppk"}, \code{"Ppu"}, \code{"Ppl"},
\code{"Z"}, or \code{"Cpy"}.}

\item{dist}{A \code{\link{pci_dist}} object (required for \code{"Cpy"};
optional otherwise).}

\item{LSL, USL}{Lower and upper specification limits.}

\item{target}{Target value (required for \code{Cpm} and \code{Cpmk}).}

\item{mu, sigma}{Short-term (within-subgroup) mean and standard deviation.}

\item{xbar, s}{Long-term (overall) mean and standard deviation.}

\item{LDL, UDL}{Lower and upper desirable limits for \code{Cpy}.}

\item{p0}{Desired yield for \code{Cpy}. If \code{LDL}/\code{UDL} are not
supplied, they are computed from \code{p0} using
\code{\link{compute_desirable_limits}}.}
}
\value{
A single numeric value, or \code{NA} with a warning when a
  one-sided index is requested but the relevant specification limit is
  \code{NULL}.
}
\description{
Computes a single process capability or performance index from the supplied
parameters.
}
\details{
The twelve supported indices are:
\describe{
  \item{Cp}{\eqn{(USL - LSL) / (6\sigma)}}
  \item{Cpk}{\eqn{\min\bigl((USL-\mu)/(3\sigma),\;(\mu-LSL)/(3\sigma)\bigr)}}
  \item{Cpu}{\eqn{(USL - \mu) / (3\sigma)}}
  \item{Cpl}{\eqn{(\mu - LSL) / (3\sigma)}}
  \item{Cpm}{\eqn{(USL - LSL) / \bigl(6\sqrt{\sigma^2+(\mu-T)^2}\bigr)}}
  \item{Cpmk}{\eqn{\min\bigl((USL-\mu),\;(\mu-LSL)\bigr) /
        \bigl(3\sqrt{\sigma^2+(\mu-T)^2}\bigr)}}
  \item{Pp}{\eqn{(USL - LSL) / (6s)}}
  \item{Ppk}{\eqn{\min\bigl((USL-\bar x)/(3s),\;(\bar x-LSL)/(3s)\bigr)}}
  \item{Ppu}{\eqn{(USL - \bar x) / (3s)}}
  \item{Ppl}{\eqn{(\bar x - LSL) / (3s)}}
  \item{Z}{\eqn{\min\bigl((USL-\mu)/\sigma,\;(\mu-LSL)/\sigma\bigr)}}
  \item{Cpy}{\eqn{p / p_0 = \bigl(F(USL)-F(LSL)\bigr) /
        \bigl(F(UDL)-F(LDL)\bigr)}}
}
}
\examples{
pci("Cp",  LSL = 57, USL = 63, sigma = 1)            # 1.0
pci("Cpk", LSL = 57, USL = 63, mu = 60, sigma = 1)   # 1.0
pci("Z",   LSL = 57, USL = 63, mu = 60, sigma = 1)   # 3.0

# Generalized Cpy
d <- pci_dist_normal(60, 1)
pci("Cpy", dist = d, LSL = 57, USL = 63, LDL = 57, UDL = 63) # 1.0

}
\seealso{
\code{\link{pci_dist}}, \code{\link{pci_ci}}, \code{\link{pci_grid}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pci_dist.R
\name{pci_dist}
\alias{pci_dist}
\title{Create a Distribution Specification for Process Capability Analysis}
\usage{
pci_dist(pdf, cdf = NULL, params = list(), support = c(-Inf, Inf))
}
\arguments{
\item{pdf}{A function \code{function(x, ...)} returning the density at
\code{x}. Additional distribution parameters are passed via \code{params}.}

\item{cdf}{A function \code{function(x, ...)} returning the cumulative
probability \eqn{P(Z \le x)}. If \code{NULL}, a numerical CDF is derived
via \code{\link[stats]{integrate}} over \code{pdf}.}

\item{params}{A named list of distribution parameters forwarded to
\code{pdf} and \code{cdf} via \code{\link{do.call}}.}

\item{support}{A length-2 numeric vector giving the lower and upper bounds
of the distribution's support (default \code{c(-Inf, Inf)}).}
}
\value{
An S3 object of class \code{"pci_dist"} with elements:
  \item{pdf_fn}{Closure that evaluates the PDF at a given point.}
  \item{cdf_fn}{Closure that evaluates the CDF at a given point.}
  \item{params}{The supplied parameter list.}
  \item{support}{The supplied support bounds.}
  \item{cdf_is_numeric}{Logical; \code{TRUE} if CDF was derived numerically.}
}
\description{
Constructs an S3 object of class \code{"pci_dist"} that encapsulates the
probability density function (PDF) and cumulative distribution function (CDF)
of a quality characteristic. This object is passed to \code{\link{pci}},
\code{\link{pci_ci}}, and \code{\link{pci_grid}} when computing generalised
indices such as \eqn{C_{py}}.
}
\examples{
# Normal distribution
d <- pci_dist(
  pdf = function(x, mean, sd) dnorm(x, mean, sd),
  cdf = function(x, mean, sd) pnorm(x, mean, sd),
  params = list(mean = 0, sd = 1)
)
d$cdf_fn(0)   # 0.5

# Weibull with only PDF (CDF derived numerically)
d2 <- pci_dist(
  pdf = function(x, shape, scale) dweibull(x, shape, scale),
  params = list(shape = 2, scale = 10),
  support = c(0, 60)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{cpmk}
\alias{cpmk}
\title{Modified Taguchi Capability Index (Cpmk)}
\usage{
cpmk(LSL, USL, mu, sigma, target)
}
\arguments{
\item{LSL}{Lower specification limit.}

\item{USL}{Upper specification limit.}

\item{mu}{Short-term process mean.}

\item{sigma}{Short-term standard deviation.}

\item{target}{Target value \eqn{T}.}
}
\value{
Numeric value of \eqn{C_{pmk}}.
}
\description{
\eqn{C_{pmk} = \min\bigl((USL-\mu),\;(\mu-LSL)\bigr) /
  \bigl(3\sqrt{\sigma^2+(\mu-T)^2}\bigr)}.
}
\examples{
cpmk(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60)   # 1.0
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{inverse_cdf_sample}
\alias{inverse_cdf_sample}
\title{Draw random samples from a pci_dist via inverse-CDF sampling}
\usage{
inverse_cdf_sample(dist, n)
}
\arguments{
\item{dist}{A \code{pci_dist} object.}

\item{n}{Number of samples to draw.}
}
\value{
Numeric vector of length \code{n}.
}
\description{
Generates samples by drawing U ~ Uniform(0,1) and inverting the CDF
using \code{\link[stats]{uniroot}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{ppu}
\alias{ppu}
\title{Upper Performance Index (Ppu)}
\usage{
ppu(USL, xbar, s)
}
\arguments{
\item{USL}{Upper specification limit.}

\item{xbar}{Long-term mean.}

\item{s}{Long-term standard deviation.}
}
\value{
Numeric value of \eqn{P_{pu}}.
}
\description{
\eqn{P_{pu} = (USL - \bar{x}) / (3s)}.
}
\examples{
ppu(USL = 63, xbar = 60, s = 1.5)   # 0.6667
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{ppk}
\alias{ppk}
\title{Performance Capability Index (Ppk)}
\usage{
ppk(LSL, USL, xbar, s)
}
\arguments{
\item{LSL}{Lower specification limit.}

\item{USL}{Upper specification limit.}

\item{xbar}{Long-term (overall) mean.}

\item{s}{Long-term standard deviation.}
}
\value{
Numeric value of \eqn{P_{pk}}.
}
\description{
\eqn{P_{pk} = \min\bigl((USL-\bar{x})/(3s),\;(\bar{x}-LSL)/(3s)\bigr)}.
}
\examples{
ppk(LSL = 57, USL = 63, xbar = 60, s = 1.5)   # 0.6667
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{validate_alpha}
\alias{validate_alpha}
\title{Validate significance level alpha}
\usage{
validate_alpha(alpha)
}
\arguments{
\item{alpha}{Significance level}
}
\value{
Invisibly returns \code{NULL}. Called for side effects (input validation).
}
\description{
Validate significance level alpha
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pci_grid.R
\name{pci_grid}
\alias{pci_grid}
\title{Sensitivity Grid of Process Capability Indices}
\usage{
pci_grid(
  index,
  dist = NULL,
  LSL = NULL,
  USL = NULL,
  target = NULL,
  sigma_vals = NULL,
  s_vals = NULL,
  p0_vals = NULL,
  mu = NULL,
  xbar = NULL,
  alpha_vals = c(0.1, 0.05, 0.01),
  n = NULL,
  B = 2000
)
}
\arguments{
\item{index}{Character: one of the 12 index names (see \code{\link{pci}}).}

\item{dist}{A \code{\link{pci_dist}} object (required for \code{"Cpy"}).}

\item{LSL, USL}{Specification limits.}

\item{target}{Target value (for \code{Cpm}/\code{Cpmk}).}

\item{sigma_vals}{Numeric vector of \eqn{\sigma} values to sweep
(for capability indices).}

\item{s_vals}{Numeric vector of \eqn{s} values to sweep
(for performance indices).}

\item{p0_vals}{Numeric vector of \eqn{p_0} values to sweep (for \code{Cpy}).}

\item{mu}{Short-term process mean.}

\item{xbar}{Long-term process mean.}

\item{alpha_vals}{Numeric vector of significance levels.
Default \code{c(0.10, 0.05, 0.01)}.}

\item{n}{Sample size for bootstrap CIs. If \code{NULL}, CIs are omitted.}

\item{B}{Bootstrap replications. Default 2000.}
}
\value{
A \code{data.frame} (with additional class \code{"pci_grid"})
  in long format with columns:
  \code{index}, \code{sigma}, \code{s}, \code{p0}, \code{alpha},
  \code{estimate}, \code{lower}, \code{upper}.
  Columns that do not apply to the chosen index contain \code{NA}.
}
\description{
Computes a PCI across a grid of parameter values (short-term standard
deviation \code{sigma}, long-term standard deviation \code{s}, and/or
desired yield \code{p0}), optionally with bootstrap confidence intervals at
multiple significance levels.
}
\examples{
grid <- pci_grid("Cp", LSL = 57, USL = 63,
                 sigma_vals = c(0.5, 1.0, 1.5),
                 mu = 60, alpha_vals = c(0.05, 0.01))
print(grid)

}
\seealso{
\code{\link{pci}}, \code{\link{pci_ci}},
  \code{\link{plot.pci_grid}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pci_ci.R
\name{pci_ci}
\alias{pci_ci}
\title{Bootstrap Confidence Interval for a Process Capability Index}
\usage{
pci_ci(
  index,
  dist,
  n,
  data = NULL,
  LSL = NULL,
  USL = NULL,
  target = NULL,
  LDL = NULL,
  UDL = NULL,
  p0 = NULL,
  alpha = 0.05,
  B = 2000,
  method = c("percentile", "bca")
)
}
\arguments{
\item{index}{Character: one of the 12 index names (see \code{\link{pci}}).}

\item{dist}{A \code{\link{pci_dist}} object.}

\item{n}{Integer sample size.}

\item{data}{Optional numeric vector of observed data. If supplied,
nonparametric bootstrap is used instead of parametric.}

\item{LSL, USL}{Specification limits.}

\item{target}{Target value (for \code{Cpm}/\code{Cpmk}).}

\item{LDL, UDL}{Desirable limits (for \code{Cpy}).}

\item{p0}{Desired yield (for \code{Cpy}).}

\item{alpha}{Significance level; the interval covers \eqn{1 - \alpha}.
Default 0.05.}

\item{B}{Number of bootstrap replications. Default 2000.}

\item{method}{Character: \code{"percentile"} (default) or \code{"bca"}.}
}
\value{
An S3 object of class \code{"pci_ci"} with components:
  \item{index}{The index name.}
  \item{estimate}{Point estimate.}
  \item{lower}{Lower CI bound.}
  \item{upper}{Upper CI bound.}
  \item{alpha}{The significance level used.}
  \item{method}{The interval method used.}
  \item{boot_values}{Numeric vector of all \code{B} bootstrap estimates.}
}
\description{
Computes a confidence interval for a PCI via parametric or nonparametric
bootstrap.
}
\details{
**Parametric bootstrap** (when \code{data = NULL}): \code{B} samples of size
\code{n} are drawn from \code{dist} via inverse-CDF sampling.
For each sample the process mean, standard deviation (and, for
\eqn{C_{py}}, the empirical CDF) are estimated, and the index is
recomputed.

**Nonparametric bootstrap** (when \code{data} is supplied): \code{B}
resamples of the same length are drawn with replacement from \code{data}.

Two interval methods are available:
\describe{
  \item{percentile}{Uses the \code{alpha/2} and \code{1 - alpha/2}
    quantiles of the bootstrap distribution.}
  \item{bca}{Bias-corrected and accelerated interval using jackknife
    acceleration and bootstrap bias correction (Efron, 1987).}
}
}
\examples{
d <- pci_dist_normal(mean = 60, sd = 1)
ci <- pci_ci("Cp", dist = d, n = 30, LSL = 57, USL = 63,
             alpha = 0.05, B = 500)
print(ci)

}
\seealso{
\code{\link{pci}}, \code{\link{pci_grid}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{cp}
\alias{cp}
\title{Process Potential Index (Cp)}
\usage{
cp(LSL, USL, sigma)
}
\arguments{
\item{LSL}{Lower specification limit.}

\item{USL}{Upper specification limit.}

\item{sigma}{Short-term (within-subgroup) standard deviation.}
}
\value{
Numeric value of \eqn{C_p}.
}
\description{
\eqn{C_p = (USL - LSL) / (6\sigma)}.
}
\examples{
cp(LSL = 57, USL = 63, sigma = 1)   # 1.0
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{cpm}
\alias{cpm}
\title{Taguchi Capability Index (Cpm)}
\usage{
cpm(LSL, USL, mu, sigma, target)
}
\arguments{
\item{LSL}{Lower specification limit.}

\item{USL}{Upper specification limit.}

\item{mu}{Short-term process mean.}

\item{sigma}{Short-term standard deviation.}

\item{target}{Target value \eqn{T}.}
}
\value{
Numeric value of \eqn{C_{pm}}.
}
\description{
\eqn{C_{pm} = (USL - LSL) / \bigl(6\sqrt{\sigma^2 + (\mu - T)^2}\bigr)}.
}
\examples{
cpm(LSL = 57, USL = 63, mu = 60, sigma = 1, target = 60)   # 1.0
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{compute_index_from_sample}
\alias{compute_index_from_sample}
\title{Compute a PCI from sample statistics}
\usage{
compute_index_from_sample(
  index,
  sample_data,
  LSL,
  USL,
  target,
  LDL,
  UDL,
  p0,
  dist
)
}
\value{
A numeric value of the calculated process capability index, or \code{NA_real_} on error/invalid input.
}
\description{
Compute a PCI from sample statistics
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{cpl}
\alias{cpl}
\title{Lower Capability Index (Cpl)}
\usage{
cpl(LSL, mu, sigma)
}
\arguments{
\item{LSL}{Lower specification limit.}

\item{mu}{Short-term process mean.}

\item{sigma}{Short-term standard deviation.}
}
\value{
Numeric value of \eqn{C_{pl}}.
}
\description{
\eqn{C_{pl} = (\mu - LSL) / (3\sigma)}.
}
\examples{
cpl(LSL = 57, mu = 60, sigma = 1)   # 1.0
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{validate_p0}
\alias{validate_p0}
\title{Validate desired yield p0}
\usage{
validate_p0(p0)
}
\arguments{
\item{p0}{Desired yield}
}
\value{
Invisibly returns \code{NULL}. Called for side effects (input validation).
}
\description{
Validate desired yield p0
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{z_level}
\alias{z_level}
\title{Sigma Level (Z)}
\usage{
z_level(LSL, USL, mu, sigma)
}
\arguments{
\item{LSL}{Lower specification limit.}

\item{USL}{Upper specification limit.}

\item{mu}{Short-term process mean.}

\item{sigma}{Short-term standard deviation.}
}
\value{
Numeric value of \eqn{Z}.
}
\description{
\eqn{Z = \min\bigl((USL-\mu)/\sigma,\;(\mu-LSL)/\sigma\bigr)}.
}
\examples{
z_level(LSL = 57, USL = 63, mu = 60, sigma = 1)   # 3.0
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pci_dist.R
\name{print.pci_dist}
\alias{print.pci_dist}
\title{Print method for pci_dist objects}
\usage{
\method{print}{pci_dist}(x, ...)
}
\arguments{
\item{x}{A \code{pci_dist} object.}

\item{...}{Additional arguments (ignored).}
}
\value{
Invisibly returns the input \code{x} of class \code{"pci_dist"}.
  Called for side effect of printing distribution parameters and specification details to the console.
}
\description{
Print method for pci_dist objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{ppl}
\alias{ppl}
\title{Lower Performance Index (Ppl)}
\usage{
ppl(LSL, xbar, s)
}
\arguments{
\item{LSL}{Lower specification limit.}

\item{xbar}{Long-term mean.}

\item{s}{Long-term standard deviation.}
}
\value{
Numeric value of \eqn{P_{pl}}.
}
\description{
\eqn{P_{pl} = (\bar{x} - LSL) / (3s)}.
}
\examples{
ppl(LSL = 57, xbar = 60, s = 1.5)   # 0.6667
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{validate_spec_limits}
\alias{validate_spec_limits}
\title{Validate specification limits}
\usage{
validate_spec_limits(LSL, USL)
}
\arguments{
\item{LSL}{Lower specification limit}

\item{USL}{Upper specification limit}
}
\value{
Invisibly returns \code{NULL}. Called for side effects (input validation).
}
\description{
Validate specification limits
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{cpu}
\alias{cpu}
\title{Upper Capability Index (Cpu)}
\usage{
cpu(USL, mu, sigma)
}
\arguments{
\item{USL}{Upper specification limit.}

\item{mu}{Short-term process mean.}

\item{sigma}{Short-term standard deviation.}
}
\value{
Numeric value of \eqn{C_{pu}}.
}
\description{
\eqn{C_{pu} = (USL - \mu) / (3\sigma)}.
}
\examples{
cpu(USL = 63, mu = 60, sigma = 1)   # 1.0
}
\seealso{
\code{\link{pci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{compute_desirable_limits}
\alias{compute_desirable_limits}
\title{Compute symmetric desirable limits for Cpy}
\usage{
compute_desirable_limits(dist, p0)
}
\arguments{
\item{dist}{A \code{pci_dist} object.}

\item{p0}{Desired yield (probability), in (0, 1].}
}
\value{
Named numeric vector \code{c(LDL = ..., UDL = ...)}.
}
\description{
Given a \code{pci_dist} and desired yield \code{p0}, computes
LDL and UDL symmetric about the median of the distribution such
that \code{F(UDL) - F(LDL) = p0}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pci_dist.R
\name{pci_dist_normal}
\alias{pci_dist_normal}
\title{Normal Distribution Shortcut}
\usage{
pci_dist_normal(mean = 0, sd = 1)
}
\arguments{
\item{mean}{Mean of the normal distribution.}

\item{sd}{Standard deviation of the normal distribution.}
}
\value{
An S3 object of class \code{"pci_dist"} encapsulating the normal
  distribution PDF and CDF functions, parameter list, and support bounds.
}
\description{
Convenience constructor for a \code{\link{pci_dist}} based on the normal
distribution. Useful for sanity-checking that \eqn{C_{py}} collapses to
the classical indices when \eqn{F} is the normal CDF.
}
\examples{
d <- pci_dist_normal(mean = 60, sd = 1)
d$cdf_fn(60)   # 0.5
d$pdf_fn(60)   # ~ 0.3989

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{cpy}
\alias{cpy}
\title{Generalized Process Capability Index (Cpy)}
\usage{
cpy(dist, LSL, USL, LDL = NULL, UDL = NULL, p0 = NULL)
}
\arguments{
\item{dist}{A \code{\link{pci_dist}} object.}

\item{LSL}{Lower specification limit.}

\item{USL}{Upper specification limit.}

\item{LDL}{Lower desirable limit (optional if \code{p0} is given).}

\item{UDL}{Upper desirable limit (optional if \code{p0} is given).}

\item{p0}{Desired yield (optional if \code{LDL}/\code{UDL} are given).}
}
\value{
Numeric value of \eqn{C_{py}}.
}
\description{
\eqn{C_{py} = p / p_0 = \bigl(F(USL)-F(LSL)\bigr) /
  \bigl(F(UDL)-F(LDL)\bigr)}, where \eqn{F} is the CDF from \code{dist}
(Maiti, Saha & Nanda, 2010).
}
\examples{
d <- pci_dist_normal(mean = 60, sd = 1)
cpy(d, LSL = 57, USL = 63, LDL = 57, UDL = 63)   # 1.0
}
\seealso{
\code{\link{pci}}, \code{\link{pci_dist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pci_ci.R
\name{print.pci_ci}
\alias{print.pci_ci}
\title{Print method for pci_ci objects}
\usage{
\method{print}{pci_ci}(x, ...)
}
\arguments{
\item{x}{A \code{pci_ci} object.}

\item{...}{Additional arguments (ignored).}
}
\value{
Invisibly returns the input \code{x} of class \code{"pci_ci"}.
  Called for side effect of printing formatted confidence interval details to the console.
}
\description{
Print method for pci_ci objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience.R
\name{pp}
\alias{pp}
\title{Performance Index (Pp)}
\usage{
pp(LSL, USL, s)
}
\arguments{
\item{LSL}{Lower specification limit.}

\item{USL}{Upper specification limit.}

\item{s}{Long-term (overall) standard deviation.}
}
\value{
Numeric value of \eqn{P_p}.
}
\description{
\eqn{P_p = (USL - LSL) / (6s)}.
}
\examples{
pp(LSL = 57, USL = 63, s = 1.5)   # 0.6667
}
\seealso{
\code{\link{pci}}
}
